@@ -8,14 +8,13 @@
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 """Test functioning of the datalad main cmdline utility """
 
-from datalad.tests.utils import known_failure_direct_mode
 from datalad.tests.utils import on_windows
 
 
 import re
 import sys
-from six.moves import StringIO
-from mock import patch
+from io import StringIO
+from unittest.mock import patch
 
 import datalad
 from ..main import (
@@ -240,6 +239,13 @@ def test_cfg_override(path):
         assert_in('datalad.dummy: this', out)
 
 
+def test_incorrect_cfg_override():
+    run_main(['-c', 'some', 'wtf'], exit_code=3)
+    run_main(['-c', 'some=', 'wtf'], exit_code=3)
+    run_main(['-c', 'some.var', 'wtf'], exit_code=3)
+    run_main(['-c', 'some.var=', 'wtf'], exit_code=3)
+
+
 def test_fail_with_short_help():
     out = StringIO()
     with assert_raises(SystemExit) as cme:
@@ -282,4 +288,4 @@ def test_fix_datalad_ri():
     assert_equal(_fix_datalad_ri('//a'), '///a')
     assert_equal(_fix_datalad_ri('///a'), '///a')
     assert_equal(_fix_datalad_ri('//a/b'), '///a/b')
-    assert_equal(_fix_datalad_ri('///a/b'), '///a/b')
\ No newline at end of file
+    assert_equal(_fix_datalad_ri('///a/b'), '///a/b')
