@@ -24,7 +24,7 @@ from datalad.tests.utils import (
     with_tempfile,
 )
 from datalad.support.exceptions import (
-    NoDatasetArgumentFound,
+    NoDatasetFound,
     IncompleteResultsError,
 )
 from datalad.distribution.dataset import Dataset
@@ -60,7 +60,7 @@ def test_status_basics(path, linkpath, otherdir):
         path = linkpath
 
     with chpwd(path):
-        assert_raises(NoDatasetArgumentFound, status)
+        assert_raises(NoDatasetFound, status)
     ds = Dataset(path).create()
     # outcome identical between ds= and auto-discovery
     with chpwd(path):
