@@ -18,11 +18,11 @@ def _get_custom_runner(d):
     """
     # We could just propagate current environ I guess to versatile our testing
     env = os.environ.copy()
-    env.update({'DATALAD_LOGTARGET': d + '_custom.log'})
-    if os.environ.get('DATALAD_LOGLEVEL'):
-        env['DATALAD_LOGLEVEL'] = os.environ.get('DATALAD_LOGLEVEL')
-    if os.environ.get('DATALAD_PROTOCOL_REMOTE'):  # TODO config tests.customremotes.protocol
+    env.update({'DATALAD_LOG_TARGET': d + '_custom.log'})
+    if os.environ.get('DATALAD_LOG_LEVEL'):
+        env['DATALAD_LOG_LEVEL'] = os.environ.get('DATALAD_LOG_LEVEL')
+    if os.environ.get('DATALAD_TESTS_PROTOCOLREMOTE'):  # TODO config tests.customremotes.protocol
         protocol = AnnexExchangeProtocol(d, 'archive')
     else:
         protocol = None
-    return Runner(cwd=d, env=env, protocol=protocol)
\ No newline at end of file
+    return Runner(cwd=d, env=env, protocol=protocol)
