@@ -14,51 +14,66 @@ __docformat__ = 'restructuredtext'
 import logging
 
 import os
-from os.path import basename
-from os.path import relpath
+import os.path as op
 
-from six.moves.urllib.parse import urlparse
+from urllib.parse import urlparse
 
 from datalad.interface.base import Interface
 from datalad.interface.utils import eval_results
 from datalad.interface.base import build_doc
 from datalad.interface.results import get_status_dict
 from datalad.support.annexrepo import AnnexRepo
-from datalad.support.constraints import EnsureStr
-from datalad.support.constraints import EnsureChoice
-from datalad.support.constraints import EnsureNone
-from datalad.support.constraints import EnsureBool
+from datalad.support.constraints import (
+    EnsureStr,
+    EnsureChoice,
+    EnsureNone,
+    EnsureBool,
+)
 from datalad.support.param import Parameter
-from datalad.support.exceptions import CommandError
-from datalad.support.exceptions import InsufficientArgumentsError
-from datalad.support.exceptions import AccessDeniedError
-from datalad.support.exceptions import AccessFailedError
-from datalad.support.exceptions import RemoteNotAvailableError
-from datalad.support.exceptions import DownloadError
-from datalad.support.network import RI
-from datalad.support.network import URL
+from datalad.support.exceptions import (
+    CommandError,
+    InsufficientArgumentsError,
+    AccessDeniedError,
+    AccessFailedError,
+    RemoteNotAvailableError,
+    DownloadError,
+)
+from datalad.support.network import (
+    RI,
+    PathRI,
+    URL,
+)
 from datalad.support.gitrepo import GitRepo
-from datalad.interface.common_opts import recursion_flag
-from datalad.interface.common_opts import recursion_limit
-from datalad.interface.common_opts import as_common_datasrc
-from datalad.interface.common_opts import publish_depends
-from datalad.interface.common_opts import publish_by_default
-from datalad.interface.common_opts import annex_wanted_opt
-from datalad.interface.common_opts import annex_required_opt
-from datalad.interface.common_opts import annex_group_opt
-from datalad.interface.common_opts import annex_groupwanted_opt
-from datalad.interface.common_opts import inherit_opt
-from datalad.interface.common_opts import location_description
+from datalad.interface.common_opts import (
+    recursion_flag,
+    recursion_limit,
+    as_common_datasrc,
+    publish_depends,
+    publish_by_default,
+    annex_wanted_opt,
+    annex_required_opt,
+    annex_group_opt,
+    annex_groupwanted_opt,
+    inherit_opt,
+    location_description,
+)
 from datalad.downloaders.credentials import UserPassword
-from datalad.distribution.dataset import require_dataset
-from datalad.distribution.dataset import Dataset
+from datalad.distribution.dataset import (
+    require_dataset,
+    Dataset,
+)
 from datalad.distribution.update import Update
-from datalad.utils import assure_list
-from datalad.utils import slash_join
+from datalad.utils import (
+    assure_list,
+    slash_join,
+    Path,
+)
 from datalad.dochelpers import exc_str
 
-from .dataset import EnsureDataset
-from .dataset import datasetmethod
+from .dataset import (
+    EnsureDataset,
+    datasetmethod,
+)
 
 lgr = logging.getLogger('datalad.distribution.siblings')
 
@@ -123,10 +138,10 @@ class Siblings(Interface):
         name=Parameter(
             args=('-s', '--name',),
             metavar='NAME',
-            doc="""name of the sibling. For sibling removal this option is
-            mandatory, otherwise the hostname part of a given URL is used as a
-            default. This option can be used to limit 'query' to a specific
-            sibling.""",
+            doc="""name of the sibling. For addition with path "URLs" and
+            sibling removal this option is mandatory, otherwise the hostname
+            part of a given URL is used as a default. This option can be used
+            to limit 'query' to a specific sibling.""",
             constraints=EnsureStr() | EnsureNone()),
         action=Parameter(
             args=('action',),
@@ -228,7 +243,7 @@ class Siblings(Interface):
 
         res_kwargs = dict(refds=refds_path, logger=lgr)
 
-        ds_name = basename(dataset.path)
+        ds_name = op.basename(dataset.path)
 
         # do not form single list of datasets (with recursion results) to
         # give fastest possible response, for the precise of a long-all
@@ -254,14 +269,16 @@ class Siblings(Interface):
         # layout?
         replicate_local_structure = url and "%NAME" not in url
 
+        subds_pushurl = None
         for subds in dataset.subdatasets(
                 fulfilled=True,
                 recursive=recursive, recursion_limit=recursion_limit,
                 result_xfm='datasets'):
-            subds_name = relpath(subds.path, start=dataset.path)
+            subds_name = op.relpath(subds.path, start=dataset.path)
             if replicate_local_structure:
                 subds_url = slash_join(url, subds_name)
-                subds_pushurl = slash_join(pushurl, subds_name)
+                if pushurl:
+                    subds_pushurl = slash_join(pushurl, subds_name)
             else:
                 subds_url = \
                     _mangle_urls(url, '/'.join([ds_name, subds_name]))
@@ -287,7 +304,7 @@ class Siblings(Interface):
         if res['status'] != 'ok' or not res.get('action', '').endswith('-sibling') :
             # logging complained about this already
             return
-        path = relpath(res['path'],
+        path = op.relpath(res['path'],
                        res['refds']) if res.get('refds', None) else res['path']
         got_url = 'url' in res
         spec = '{}{}{}{}'.format(
@@ -330,7 +347,12 @@ def _add_remote(
     if not name:
         urlri = RI(url)
         # use the hostname as default remote name
-        name = urlri.hostname
+        try:
+            name = urlri.hostname
+        except AttributeError:
+            raise InsufficientArgumentsError(
+                "cannot derive a default remote name from '{}', "
+                "please specify a name.".format(url))
         lgr.debug(
             "No sibling name given, use URL hostname '%s' as sibling name",
             name)
@@ -347,6 +369,10 @@ def _add_remote(
             message=("sibling is already known: %s, use `configure` instead?", name),
             **res_kwargs)
         return
+    if isinstance(RI(url), PathRI):
+        # make sure any path URL is stored in POSIX conventions for consistency
+        # with git's behavior (e.g. origin configured by clone)
+        url = Path(url).as_posix()
     # this remote is fresh: make it known
     # just minimalistic name and URL, the rest is coming from `configure`
     ds.repo.add_remote(name, url)
@@ -427,8 +453,7 @@ def _configure_remote(
         if fetch:
             # fetch the remote so we are up to date
             for r in Update.__call__(
-                    dataset=res_kwargs['refds'],
-                    path=[dict(path=ds.path, type='dataset')],
+                    dataset=ds.path,
                     sibling=name,
                     merge=False,
                     recursive=False,
@@ -450,7 +475,8 @@ def _configure_remote(
             # makes sense only if current AND super are annexes, so it is
             # kinda a boomer, since then forbids having a super a pure git
             if isinstance(ds.repo, AnnexRepo) and \
-                    isinstance(delayed_super.repo, AnnexRepo):
+                    isinstance(delayed_super.repo, AnnexRepo) and \
+                    name in delayed_super.repo.get_remotes():
                 if annex_wanted is None:
                     annex_wanted = _inherit_annex_var(
                         delayed_super, name, 'wanted')
@@ -629,7 +655,7 @@ def _query_remotes(
                               if k.startswith('remote.{}.'.format(remote))]:
                 info[remotecfg[8 + len(remote):]] = ds.config[remotecfg]
         if get_annex_info and info.get('annex-uuid', None):
-            ainfo = annex_info.get(info['annex-uuid'])
+            ainfo = annex_info.get(info['annex-uuid'], {})
             annex_description = ainfo.get('description', None)
             if annex_description is not None:
                 info['annex-description'] = annex_description
