@@ -10,32 +10,35 @@
 """
 
 import os
-import shutil
-from os.path import join as opj, abspath, normpath, relpath, exists
+import os.path as op
+from os.path import join as opj, abspath, relpath
 
-from ..dataset import Dataset, EnsureDataset, resolve_path, require_dataset
+
+from ..dataset import Dataset, EnsureDataset, require_dataset
+from ..dataset import resolve_path
 from datalad import cfg
 from datalad.api import create
 from datalad.api import get
-from datalad.utils import chpwd, getpwd, rmtree
+import datalad.utils as ut
+from datalad.utils import chpwd, rmtree
 from datalad.utils import _path_
-from datalad.utils import get_dataset_root
 from datalad.utils import on_windows
+from datalad.utils import Path
 from datalad.support.gitrepo import GitRepo
 from datalad.support.annexrepo import AnnexRepo
 
 from nose.tools import ok_, eq_, assert_false, assert_equal, assert_true, \
     assert_is_instance, assert_is_none, assert_is_not, assert_is_not_none
 from datalad.tests.utils import SkipTest
-from datalad.tests.utils import with_tempfile, assert_in, with_tree, with_testrepos
-from datalad.tests.utils import assert_cwd_unchanged
+from datalad.tests.utils import with_tempfile, with_testrepos
 from datalad.tests.utils import assert_raises
 from datalad.tests.utils import known_failure_windows
 from datalad.tests.utils import assert_is
 from datalad.tests.utils import assert_not_equal
+from datalad.tests.utils import assert_result_count
+from datalad.tests.utils import OBSCURE_FILENAME
 
 from datalad.support.exceptions import InsufficientArgumentsError
-from datalad.support.exceptions import PathKnownToRepositoryError
 
 
 def test_EnsureDataset():
@@ -48,45 +51,19 @@ def test_EnsureDataset():
     assert_raises(ValueError, c, (1, 2, 3))
     assert_raises(ValueError, c, {"what": "ever"})
 
-    # returns Dataset, when string or Dataset passed
-    res = c(opj("some", "path"))
-    ok_(isinstance(res, Dataset))
-    ok_(isinstance(c(res), Dataset))
-    ok_(c(res) is res)
+    # let's a Dataset instance pass, but leaves a path untouched
+    test_path = opj("some", "path")
+    ok_(isinstance(c(test_path), type(test_path)))
+    ok_(isinstance(Dataset(test_path), Dataset))
 
     # Note: Ensuring that string is valid path is not
     # part of the constraint itself, so not explicitly tested here.
 
 
-@assert_cwd_unchanged
-@with_tempfile(mkdir=True)
-def test_resolve_path(somedir):
-
-    abs_path = abspath(somedir)  # just to be sure
-    rel_path = "some"
-    expl_path_cur = opj(os.curdir, rel_path)
-    expl_path_par = opj(os.pardir, rel_path)
-
-    eq_(resolve_path(abs_path), abs_path)
-
-    current = getpwd()
-    # no Dataset => resolve using cwd:
-    eq_(resolve_path(abs_path), abs_path)
-    eq_(resolve_path(rel_path), opj(current, rel_path))
-    eq_(resolve_path(expl_path_cur), normpath(opj(current, expl_path_cur)))
-    eq_(resolve_path(expl_path_par), normpath(opj(current, expl_path_par)))
-
-    # now use a Dataset as reference:
-    ds = Dataset(abs_path)
-    eq_(resolve_path(abs_path, ds), abs_path)
-    eq_(resolve_path(rel_path, ds), opj(abs_path, rel_path))
-    eq_(resolve_path(expl_path_cur, ds), normpath(opj(current, expl_path_cur)))
-    eq_(resolve_path(expl_path_par, ds), normpath(opj(current, expl_path_par)))
-
-
 # TODO: test remember/recall more extensive?
 
-
+# https://github.com/datalad/datalad/pull/3975/checks?check_run_id=369789022#step:8:531
+@known_failure_windows
 @with_testrepos('submodule_annex')
 @with_tempfile(mkdir=True)
 def test_is_installed(src, path):
@@ -103,12 +80,16 @@ def test_is_installed(src, path):
     # subdataset path.
     # Note: Unfortunately we would still be able to generate it under
     # subdirectory within submodule, e.g. `subm 1/subdir` but that is
-    # not checked here. `rev-create` will provide that protection
-    # when create/rev-create merge.
-    # Unfortunately such protection does not work in direct mode
-    if not ds.repo.is_direct_mode():
-        with assert_raises(PathKnownToRepositoryError):
-            subds.create()
+    # not checked here. `create` provides that protection though.
+    res = subds.create(on_failure='ignore',
+                       return_type='list',
+                       result_filter=None,
+                       result_xfm=None)
+    assert_result_count(res, 1)
+    assert_result_count(
+        res, 1, status='error', path=subds.path,
+        message=('collision with %s (dataset) in dataset %s',
+                 subds.path, ds.path))
     # get the submodule
     # This would init so there is a .git file with symlink info, which is
     # as we agreed is more pain than gain, so let's use our install which would
@@ -127,7 +108,7 @@ def test_is_installed(src, path):
 def test_dataset_contructor(path):
     # dataset needs a path
     assert_raises(TypeError, Dataset)
-    assert_raises(AttributeError, Dataset, None)
+    assert_raises(ValueError, Dataset, None)
     dsabs = Dataset(path)
     # always abspath
     ok_(os.path.isabs(dsabs.path))
@@ -167,15 +148,14 @@ def test_subdatasets(path):
     # from scratch
     ds = Dataset(path)
     assert_false(ds.is_installed())
-    eq_(ds.subdatasets(), [])
+    assert_raises(ValueError, ds.subdatasets)
     ds = ds.create()
     assert_true(ds.is_installed())
     eq_(ds.subdatasets(), [])
     # create some file and commit it
     open(os.path.join(ds.path, 'test'), 'w').write('some')
-    ds.add(path='test')
+    ds.save(path='test', message="Hello!", version_tag=1)
     assert_true(ds.is_installed())
-    ds.save("Hello!", version_tag=1)
     # Assuming that tmp location was not under a super-dataset
     eq_(ds.get_superdataset(), None)
     eq_(ds.get_superdataset(topmost=True), ds)
@@ -192,7 +172,7 @@ def test_subdatasets(path):
     eq_(subds.path, ds.subdatasets(result_xfm='paths')[0])
     eq_(subdss, ds.subdatasets(recursive=True))
     eq_(subdss, ds.subdatasets(fulfilled=True))
-    ds.save("with subds", version_tag=2)
+    ds.save(message="with subds", version_tag=2)
     ds.recall_state(1)
     assert_true(ds.is_installed())
     eq_(ds.subdatasets(), [])
@@ -224,9 +204,43 @@ def test_subdatasets(path):
 
     # TODO actual submodule checkout is still there
 
+    # Test ^. (the dataset for curdir) shortcut
+    # At the top should point to the top
+    with chpwd(ds.path):
+        dstop = Dataset('^.')
+        eq_(dstop, ds)
+
+    # and still does within subdir
+    os.mkdir(opj(ds.path, 'subdir'))
+    with chpwd(opj(ds.path, 'subdir')):
+        dstop = Dataset('^.')
+        eq_(dstop, ds)
+
+    # within submodule will point to submodule
+    with chpwd(subsubds.path):
+        dstop = Dataset('^.')
+        eq_(dstop, subsubds)
+
 
 @with_tempfile(mkdir=True)
-def test_require_dataset(path):
+def test_hat_dataset_more(path):
+    # from scratch
+    ds = Dataset(path).create()
+    # add itself as a subdataset (crazy, isn't it?)
+    subds = ds.install(
+        'subds', source=path,
+        result_xfm='datasets', return_type='item-or-list')
+    # must find its way all the way up from an untracked dir in a subsubds
+    untracked_subdir = op.join(subds.path, 'subdir')
+    os.makedirs(untracked_subdir)
+    with chpwd(untracked_subdir):
+        eq_(Dataset('^'), ds)
+
+
+@with_tempfile(mkdir=True)
+def check_require_dataset(ds_path, topdir):
+    path = opj(topdir, ds_path)
+    os.mkdir(path)
     with chpwd(path):
         assert_raises(
             InsufficientArgumentsError,
@@ -248,6 +262,11 @@ def test_require_dataset(path):
             check_installed=True)
 
 
+def test_require_dataset():
+    yield check_require_dataset, "simple-path"
+    yield check_require_dataset, OBSCURE_FILENAME
+
+
 @with_tempfile(mkdir=True)
 def test_dataset_id(path):
     ds = Dataset(path)
@@ -442,3 +461,107 @@ def test_symlinked_dataset_properties(repo1, repo2, repo3, non_repo, symlink):
     assert_is_not(ds_link.config, ar3.config)
     assert_false(ds_link._cfg_bound)
     assert_is_none(ds_link.id)
+
+
+@with_tempfile(mkdir=True)
+def test_resolve_path(path):
+    if op.realpath(path) != path:
+        raise SkipTest("Test assumptions require non-symlinked parent paths")
+    # initially ran into on OSX https://github.com/datalad/datalad/issues/2406
+    opath = op.join(path, "origin")
+    os.makedirs(opath)
+    if not on_windows:
+        lpath = op.join(path, "linked")
+        os.symlink('origin', lpath)
+
+    ds_global = Dataset(path)
+    # path resolution of absolute paths is not influenced by symlinks
+    # ignore the linked path on windows, it is not a symlink in the POSIX sense
+    for d in (opath,) if on_windows else (opath, lpath):
+        ds_local = Dataset(d)
+        # no symlink resolution
+        eq_(str(resolve_path(d)), d)
+        # list comes out as a list
+        eq_(resolve_path([d]), [Path(d)])
+        # multiple OK
+        eq_(resolve_path([d, d]), [Path(d), Path(d)])
+
+        with chpwd(d):
+            # be aware: knows about cwd, but this CWD has symlinks resolved
+            eq_(str(resolve_path(d).cwd()), opath)
+            # using pathlib's `resolve()` will resolve any
+            # symlinks
+            # also resolve `opath`, as on old windows systems the path might
+            # come in crippled (e.g. C:\Users\MIKE~1/...)
+            # and comparison would fails unjustified
+            eq_(resolve_path('.').resolve(), ut.Path(opath).resolve())
+            # no norming, but absolute paths, without resolving links
+            eq_(resolve_path('.'), ut.Path(d))
+            eq_(str(resolve_path('.')), d)
+
+            # there is no concept of an "explicit" relative path anymore
+            # relative is relative, regardless of the specific syntax
+            eq_(resolve_path(op.join(os.curdir, 'bu'), ds=ds_global),
+                ds_global.pathobj / 'bu')
+            # there is no full normpath-ing or other funky resolution of
+            # parent directory back-reference
+            eq_(str(resolve_path(op.join(os.pardir, 'bu'), ds=ds_global)),
+                op.join(ds_global.path, os.pardir, 'bu'))
+
+        # resolve against a dataset given as a path/str
+        # (cmdline input scenario)
+        eq_(resolve_path('bu', ds=ds_local.path), Path.cwd() / 'bu')
+        eq_(resolve_path('bu', ds=ds_global.path), Path.cwd() / 'bu')
+        # resolve against a dataset given as a dataset instance
+        # (object method scenario)
+        eq_(resolve_path('bu', ds=ds_local), ds_local.pathobj / 'bu')
+        eq_(resolve_path('bu', ds=ds_global), ds_global.pathobj / 'bu')
+        # not being inside a dataset doesn't change the resolution result
+        eq_(resolve_path(op.join(os.curdir, 'bu'), ds=ds_global),
+            ds_global.pathobj / 'bu')
+        eq_(str(resolve_path(op.join(os.pardir, 'bu'), ds=ds_global)),
+            op.join(ds_global.path, os.pardir, 'bu'))
+
+
+# little brother of the test above, but actually (must) run
+# under any circumstances
+@with_tempfile(mkdir=True)
+def test_resolve_path_symlink_edition(path):
+    deepest = ut.Path(path) / 'one' / 'two' / 'three'
+    deepest_str = str(deepest)
+    os.makedirs(deepest_str)
+    with chpwd(deepest_str):
+        # direct absolute
+        eq_(deepest, resolve_path(deepest))
+        eq_(deepest, resolve_path(deepest_str))
+        # explicit direct relative
+        eq_(deepest, resolve_path('.'))
+        eq_(deepest, resolve_path(op.join('.', '.')))
+        eq_(deepest, resolve_path(op.join('..', 'three')))
+        eq_(deepest, resolve_path(op.join('..', '..', 'two', 'three')))
+        eq_(deepest, resolve_path(op.join('..', '..', '..',
+                                              'one', 'two', 'three')))
+        # weird ones
+        eq_(deepest, resolve_path(op.join('..', '.', 'three')))
+        eq_(deepest, resolve_path(op.join('..', 'three', '.')))
+        eq_(deepest, resolve_path(op.join('..', 'three', '.')))
+        eq_(deepest, resolve_path(op.join('.', '..', 'three')))
+
+
+@with_tempfile(mkdir=True)
+def test_hashable(path):
+    path = ut.Path(path)
+    tryme = set()
+    # is it considered hashable at all
+    tryme.add(Dataset(path / 'one'))
+    eq_(len(tryme), 1)
+    # do another one, same class different path
+    tryme.add(Dataset(path / 'two'))
+    eq_(len(tryme), 2)
+    # test whether two different types of repo instances pointing
+    # to the same repo on disk are considered different
+    Dataset(path).create()
+    tryme.add(GitRepo(path))
+    eq_(len(tryme), 3)
+    tryme.add(AnnexRepo(path))
+    eq_(len(tryme), 4)
