@@ -10,11 +10,12 @@
 """
 
 
-from datalad.tests.utils import known_failure_direct_mode
 
 from os import curdir
+import os.path as op
 from os.path import join as opj, basename
 from glob import glob
+from unittest.mock import patch
 
 from datalad.api import create
 from datalad.api import get
@@ -40,6 +41,7 @@ from datalad.tests.utils import assert_result_count
 from datalad.tests.utils import assert_message
 from datalad.tests.utils import serve_path_via_http
 from datalad.tests.utils import slow
+from datalad.tests.utils import known_failure_windows
 from datalad.utils import with_pathsep
 from datalad.utils import chpwd
 from datalad.utils import assure_list
@@ -58,16 +60,18 @@ def _make_dataset_hierarchy(path):
     with open(opj(origin_sub3.path, 'file_in_annex.txt'), "w") as f:
         f.write('content3')
     origin_sub4 = origin_sub3.create('sub4')
-    origin.add('.', recursive=True)
+    origin.save(recursive=True)
     return origin, origin_sub1, origin_sub2, origin_sub3, origin_sub4
 
 
 @with_tempfile
 @with_tempfile
-def test_get_flexible_source_candidates_for_submodule(t, t2):
+@with_tempfile
+def test_get_flexible_source_candidates_for_submodule(t, t2, t3):
     f = _get_flexible_source_candidates_for_submodule
     # for now without mocking -- let's just really build a dataset
     ds = create(t)
+    sub = ds.create('sub')
     clone = install(
         t2, source=t,
         result_xfm='datasets', return_type='item-or-list')
@@ -75,22 +79,76 @@ def test_get_flexible_source_candidates_for_submodule(t, t2):
     # first one could just know about itself or explicit url provided
     sshurl = 'ssh://e.c'
     httpurl = 'http://e.c'
-    # Expansion with '/.git' no longer done in this helper
-    #sm_httpurls = [httpurl, httpurl + '/.git']
-    sm_httpurls = [httpurl]
-    eq_(f(ds, 'sub'), [])
-    eq_(f(ds, 'sub', sshurl), [sshurl])
-    eq_(f(ds, 'sub', httpurl), sm_httpurls)
-    eq_(f(ds, 'sub', None), [])  # otherwise really we have no clue were to get from
+    ds_subpath = str(ds.pathobj / 'sub')
+    eq_(f(ds, dict(path=ds_subpath, parentds=ds.path)), [])
+    eq_(f(ds, dict(path=ds_subpath, parentds=ds.path, gitmodule_url=sshurl)),
+        [('local', sshurl)])
+    eq_(f(ds, dict(path=ds_subpath, parentds=ds.path, gitmodule_url=httpurl)),
+        [('local', httpurl)])
 
     # but if we work on dsclone then it should also add urls deduced from its
     # own location default remote for current branch
-    eq_(f(clone, 'sub'), [t + '/sub'])
-    eq_(f(clone, 'sub', sshurl), [t + '/sub', sshurl])
-    eq_(f(clone, 'sub', httpurl), [t + '/sub'] + sm_httpurls)
-    eq_(f(clone, 'sub'), [t + '/sub'])  # otherwise really we have no clue were to get from
-    # TODO: check that http:// urls for the dataset itself get resolved
+    eq_(f(clone, dict(path=ds_subpath, parentds=t)), [('origin', ds_subpath)])
+    eq_(f(clone, dict(path=ds_subpath, parentds=t, gitmodule_url=sshurl)),
+        [('origin', ds_subpath), ('origin', sshurl)])
+    eq_(f(clone, dict(path=ds_subpath, parentds=t, gitmodule_url=httpurl)),
+        [('origin', ds_subpath), ('origin', httpurl)])
 
+    # make sure it does meaningful things in an actual clone with an actual
+    # record of a subdataset
+    clone_subpath = str(clone.pathobj / 'sub')
+    eq_(f(clone, clone.subdatasets(return_type='item-or-list')),
+        [
+            ('origin', ds_subpath),
+            ('local', clone_subpath),
+    ])
+
+    # check that a configured remote WITHOUT the desired submodule commit
+    # does not show up as a candidate
+    clone.siblings('add', name='myremote', url='http://example.com',
+                   result_renderer='disabled')
+    eq_(f(clone, clone.subdatasets(return_type='item-or-list')),
+        [
+            ('origin', ds_subpath),
+            ('local', clone_subpath),
+    ])
+    # inject a source URL config, should alter the result accordingly
+    with patch.dict(
+            'os.environ',
+            {'DATALAD_GET_SUBDATASET__SOURCE__CANDIDATE__BANG': 'youredead'}):
+        eq_(f(clone, clone.subdatasets(return_type='item-or-list')),
+            [
+                ('origin', ds_subpath),
+                ('subdataset-source-candidate-bang', 'youredead'),
+                ('local', clone_subpath),
+        ])
+    # verify template instantiation works
+    with patch.dict(
+            'os.environ',
+            {'DATALAD_GET_SUBDATASET__SOURCE__CANDIDATE__BANG': 'pre-{id}-post'}):
+        eq_(f(clone, clone.subdatasets(return_type='item-or-list')),
+            [
+                ('origin', ds_subpath),
+                ('subdataset-source-candidate-bang', 'pre-{}-post'.format(sub.id)),
+                ('local', clone_subpath),
+        ])
+    # now again, but have an additional remote besides origin that
+    # actually has the relevant commit
+    clone3 = install(
+        t3, source=t2,
+        result_xfm='datasets', return_type='item-or-list')
+    clone3.siblings('add', name='myremote', url=ds.path,
+                    result_renderer='disabled')
+    clone3.update(sibling='myremote')
+    # we should end up with this additional piece
+    # we are not checking for the name of the remote, because it is actually
+    # registered under two different names
+    assert_in(
+        ds_subpath,
+        [i[1] for i in f(clone3, clone3.subdatasets(return_type='item-or-list'))]
+    )
+
+    # TODO: check that http:// urls for the dataset itself get resolved
     # TODO: many more!!
 
 
@@ -102,17 +160,18 @@ def test_get_invalid_call(path, file_outside):
     assert_raises(InsufficientArgumentsError, get, None)
     assert_raises(InsufficientArgumentsError, get, [])
     # invalid dataset:
-    assert_status('impossible', get(None, dataset=path, on_failure='ignore'))
+    assert_raises(ValueError, get, None, dataset=path, on_failure='ignore')
 
     # have a plain git:
     ds = Dataset(path)
     ds.create(no_annex=True)
     with open(opj(path, "some.txt"), "w") as f:
         f.write("whatever")
-    ds.add("some.txt", to_git=True)
-    ds.save("Initial commit.")
+    ds.save("some.txt", to_git=True, message="Initial commit.")
 
-    # make it an annex:
+    # make it an annex (remove indicator file that create has placed
+    # in the dataset to make it possible):
+    (ds.pathobj / '.noannex').unlink()
     AnnexRepo(path, init=True, create=True)
     # call get again on a file in git:
     result = ds.get("some.txt")
@@ -122,7 +181,7 @@ def test_get_invalid_call(path, file_outside):
     # yoh:  but now we would need to add it to annex since clever code first
     # checks what needs to be fetched at all
     create_tree(path, {'annexed.dat': 'some'})
-    ds.add("annexed.dat")
+    ds.save("annexed.dat")
     ds.repo.drop("annexed.dat", options=['--force'])
     with assert_raises(RemoteNotAvailableError) as ce:
         ds.get("annexed.dat", source='MysteriousRemote')
@@ -133,9 +192,10 @@ def test_get_invalid_call(path, file_outside):
     assert_message("path does not exist", res)
 
     # path outside repo errors as with most other commands:
-    res = ds.get(file_outside, on_failure='ignore')
+    res = ds.get(file_outside, on_failure='ignore', result_renderer='default')
     assert_in_results(
-        res, status='impossible', message='path not associated with any dataset')
+        res, status='error',
+        message=('path not associated with dataset %s', ds))
 
 
 @with_testrepos('basic_annex', flavors='clone')
@@ -169,8 +229,7 @@ def test_get_multiple_files(path, url, ds_dir):
 
     # prepare origin
     origin = Dataset(path).create(force=True)
-    origin.add(file_list)
-    origin.save("initial")
+    origin.save(file_list, message="initial")
 
     ds = install(
         ds_dir, source=path,
@@ -209,7 +268,7 @@ def test_get_recurse_dirs(o_path, c_path):
 
     # prepare source:
     origin = Dataset(o_path).create(force=True)
-    origin.add('.')
+    origin.save()
 
     ds = install(
         c_path, source=o_path,
@@ -225,7 +284,6 @@ def test_get_recurse_dirs(o_path, c_path):
     ok_(not any(ds.repo.file_has_content(file_list)))
 
     result = ds.get('subdir')
-
     # check result:
     assert_status('ok', result)
     eq_(set([item.get('path')[len(ds.path) + 1:] for item in result
@@ -242,6 +300,8 @@ def test_get_recurse_dirs(o_path, c_path):
     ok_(ds.repo.file_has_content('file1.txt') is True)
 
 
+# https://github.com/datalad/datalad/pull/3975/checks?check_run_id=369789022#step:8:541
+@known_failure_windows
 @slow  # 15.1496s
 @with_testrepos('submodule_annex', flavors='local')
 @with_tempfile(mkdir=True)
@@ -276,8 +336,17 @@ def test_get_recurse_subdatasets(src, path):
     # MIH: Nope, we fulfill the dataset handle, but that doesn't
     #      imply fulfilling all file handles
     result = ds.get(rel_path_sub1, recursive=True)
-    # all good actions
-    assert_status('ok', result)
+    # the subdataset was already present
+    assert_in_results(
+        result,
+        type='dataset',
+        path=subds1.path,
+        status='notneeded')
+    # we got the file
+    assert_in_results(
+        result,
+        path=opj(ds.path, rel_path_sub1),
+        status='ok')
 
     assert_in_results(result, path=opj(ds.path, rel_path_sub1), status='ok')
     ok_(subds1.repo.file_has_content('test-annex.dat') is True)
@@ -310,8 +379,9 @@ def test_get_recurse_subdatasets(src, path):
     # now, the very same call, but without recursive:
     result = ds.get('.', recursive=False)
     assert_status('ok', result)
-    # one report is on the requested dir
-    eq_(len(result) - 1, 1)
+    # no duplicate reporting on subdataset install and annex-get of its
+    # directory
+    eq_(len(result), 1)
     assert_result_count(
         result, 1, path=opj(ds.path, 'test-annex.dat'), status='ok')
     ok_(ds.repo.file_has_content('test-annex.dat') is True)
@@ -369,7 +439,6 @@ def test_get_install_missing_subdataset(src, path):
 #                  'subds': {'file_in_annex.txt': 'content'}})
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
-@known_failure_direct_mode  #FIXME
 def test_get_mixed_hierarchy(src, path):
 
     origin = Dataset(src).create(no_annex=True)
@@ -378,8 +447,8 @@ def test_get_mixed_hierarchy(src, path):
         f.write('no idea')
     with open(opj(origin_sub.path, 'file_in_annex.txt'), "w") as f:
         f.write('content')
-    origin.add('file_in_git.txt', to_git=True)
-    origin_sub.add('file_in_annex.txt')
+    origin.save('file_in_git.txt', to_git=True)
+    origin_sub.save('file_in_annex.txt')
     origin.save()
 
     # now, install that thing:
@@ -397,23 +466,6 @@ def test_get_mixed_hierarchy(src, path):
     ok_(subds.repo.file_has_content("file_in_annex.txt") is True)
 
 
-@with_testrepos('submodule_annex', flavors='local')
-@with_tempfile(mkdir=True)
-def test_autoresolve_multiple_datasets(src, path):
-    with chpwd(path):
-        ds1 = install(
-            'ds1', source=src,
-            result_xfm='datasets', return_type='item-or-list')
-        ds2 = install(
-            'ds2', source=src,
-            result_xfm='datasets', return_type='item-or-list')
-        results = get([opj('ds1', 'test-annex.dat')] + glob(opj('ds2', '*.dat')))
-        # each ds has one file
-        assert_result_count(results, 2, type='file', action='get', status='ok')
-        ok_(ds1.repo.file_has_content('test-annex.dat') is True)
-        ok_(ds2.repo.file_has_content('test-annex.dat') is True)
-
-
 @slow  # 20 sec
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
@@ -424,14 +476,15 @@ def test_get_autoresolve_recurse_subdatasets(src, path):
     origin_subsub = origin_sub.create('subsub')
     with open(opj(origin_subsub.path, 'file_in_annex.txt'), "w") as f:
         f.write('content')
-    origin.add('.', recursive=True)
+    origin.save(recursive=True)
 
     ds = install(
         path, source=src,
         result_xfm='datasets', return_type='item-or-list')
     eq_(len(ds.subdatasets(fulfilled=True)), 0)
 
-    results = get(opj(ds.path, 'sub'), recursive=True, result_xfm='datasets')
+    with chpwd(ds.path):
+        results = get(opj(ds.path, 'sub'), recursive=True, result_xfm='datasets')
     eq_(len(ds.subdatasets(fulfilled=True, recursive=True)), 2)
     subsub = Dataset(opj(ds.path, 'sub', 'subsub'))
     ok_(subsub.is_installed())
@@ -493,7 +546,8 @@ def test_get_in_unavailable_subdataset(src, path):
         result_xfm='datasets', return_type='item-or-list')
     targetpath = opj('sub1', 'sub2')
     targetabspath = opj(root.path, targetpath)
-    res = get(targetabspath)
+    with chpwd(path):
+        res = get(targetabspath)
     assert_result_count(res, 2, status='ok', action='install', type='dataset')
     # dry-fit result filter that only returns the result that matched the requested
     # path
@@ -506,3 +560,31 @@ def test_get_in_unavailable_subdataset(src, path):
     ok_(sub2.is_installed())
     ok_(sub2.repo.file_has_content('file_in_annex.txt') is True)
     ok_(not Dataset(opj(targetabspath, 'sub3')).is_installed())
+
+
+@with_tempfile(mkdir=True)
+@with_tempfile(mkdir=True)
+def test_gh3356(src, path):
+    # create toy version of gh-3356 scenario
+    origin = Dataset(src).create()
+    origin_sub = origin.create(origin.pathobj / 'subdir'/ 'subds')
+    for p in (
+            (origin_sub.pathobj / 'data' / 'file_in_annex.txt'),
+            (origin_sub.pathobj / 'data' / 'file_in_annex2.txt')):
+        p.parent.mkdir(parents=True, exist_ok=True)
+        p.write_text(p.name)
+    origin.save(recursive=True)
+    clone = install(
+        path, source=src, result_xfm='datasets', return_type='item-or-list')
+    targetpaths = [
+        opj('subdir', 'subds', 'data', 'file_in_annex.txt'),
+        opj('subdir', 'subds', 'data', 'file_in_annex2.txt'),
+    ]
+    with chpwd(path):
+        res = get(targetpaths)
+    # get() must report success on two files
+    assert_result_count(res, 2, action='get', type='file', status='ok')
+    # status must report content for two files
+    assert_result_count(
+        clone.status(recursive=True, annex='all', report_filetype='eval'), 2,
+        action='status', has_content=True)
