@@ -12,14 +12,9 @@
 
 __docformat__ = 'restructuredtext'
 
-from six.moves.urllib.parse import urlparse
-
-
-from .providers import Providers
-
 from logging import getLogger
 lgr = getLogger('datalad.providers')
 
 # TODO: we might not need to instantiate it right here
 # lgr.debug("Initializing data providers credentials interface")
-# providers = Providers().from_config_files()
\ No newline at end of file
+# providers = Providers().from_config_files()
