@@ -16,7 +16,6 @@ __docformat__ = 'restructuredtext'
 
 import re
 import os
-import shlex
 import tempfile
 
 from os.path import join as opj, realpath, curdir, exists, lexists, relpath, basename
@@ -42,11 +41,10 @@ from ..utils import getpwd, rmtree, file_basename
 from ..utils import md5sum
 from ..utils import assure_tuple_or_list
 from ..utils import get_dataset_root
+from ..utils import split_cmdline
 
 from datalad.customremotes.base import init_datalad_remote
 
-from six import string_types
-
 from ..log import logging
 lgr = logging.getLogger('datalad.interfaces.add_archive_content')
 
@@ -315,8 +313,8 @@ class AddArchiveContent(Interface):
             annex.always_commit = annex.fake_dates_enabled
 
             if annex_options:
-                if isinstance(annex_options, string_types):
-                    annex_options = shlex.split(annex_options)
+                if isinstance(annex_options, str):
+                    annex_options = split_cmdline(annex_options)
 
             leading_dir = earchive.get_leading_directory(
                 depth=leading_dirs_depth, exclude=exclude, consider=leading_dirs_consider) \
@@ -510,7 +508,8 @@ class AddArchiveContent(Interface):
                 commit_stats = outside_stats if outside_stats else stats
                 annex.precommit()  # so batched ones close and files become annex symlinks etc
                 precommitted = True
-                if annex.is_dirty(untracked_files=False):
+                if any(r.get('state', None) != 'clean'
+                       for p, r in annex.status(untracked='no').items()):
                     annex.commit(
                         "Added content extracted from %s %s\n\n%s" %
                         (origin, archive_rpath, commit_stats.as_str(mode='full')),
