@@ -39,7 +39,6 @@ from datalad.interface.common_opts import recursion_flag
 from datalad.interface.common_opts import recursion_limit
 
 from datalad.distribution.dataset import Dataset
-from datalad.distribution.dataset import resolve_path
 from datalad.distribution.dataset import EnsureDataset
 from datalad.distribution.dataset import datasetmethod
 
@@ -48,6 +47,10 @@ from datalad.utils import with_pathsep as _with_sep
 from datalad.utils import path_startswith
 from datalad.utils import path_is_subpath
 from datalad.utils import assure_list
+from datalad.utils import dlabspath
+from datalad.utils import expandpath
+from datalad.utils import is_explicit_path
+from datalad.utils import getpwd
 
 from datalad.consts import PRE_INIT_COMMIT_SHA
 
@@ -55,6 +58,34 @@ from datalad.consts import PRE_INIT_COMMIT_SHA
 lgr = logging.getLogger('datalad.interface.annotate_paths')
 
 
+# former main implementation, no a left-over that is only used in this
+# obsolete source file
+def _resolve_path(path, ds=None):
+    """Resolve a path specification (against a Dataset location)
+
+    Any explicit path (absolute or relative) is returned as an absolute path.
+    In case of an explicit relative path, the current working directory is
+    used as a reference. Any non-explicit relative path is resolved against
+    as dataset location, i.e. considered relative to the location of the
+    dataset. If no dataset is provided, the current working directory is
+    used.
+
+    Returns
+    -------
+    Absolute path
+    """
+    path = expandpath(path, force_absolute=False)
+    if is_explicit_path(path):
+        # normalize path consistently between two (explicit and implicit) cases
+        return dlabspath(path, norm=True)
+
+    # no dataset given, use CWD as reference
+    # note: abspath would disregard symlink in CWD
+    top_path = getpwd() \
+        if ds is None else ds.path if isinstance(ds, Dataset) else ds
+    return normpath(opj(top_path, path))
+
+
 def annotated2content_by_ds(annotated, refds_path):
     """Helper to convert annotated paths into an old-style content_by_ds dict
 
@@ -177,7 +208,12 @@ def get_modified_subpaths(aps, refds, revision, recursion_limit=None,
 
     # life is simple: we diff the base dataset
     modified = []
-    for r in refds.diff(
+    # Diff.__call__ is used to get access to the now obsolete interface.diff
+    # that exists merely for annotate_paths. (refds.diff corresponds to
+    # core.local.diff.)
+    from datalad.interface.diff import Diff
+    for r in Diff.__call__(
+            dataset=refds,
             # we cannot really limit the diff paths easily because we might get
             # or miss content (e.g. subdatasets) if we don't figure out which
             # ones are known -- and we don't want that
@@ -274,7 +310,7 @@ def get_modified_subpaths(aps, refds, revision, recursion_limit=None,
 def rawpath2ap(path, refds_path):
     orig_path_request = path
     # this is raw, resolve
-    path = resolve_path(path, refds_path)
+    path = _resolve_path(path, refds_path)
     # collect info on this path
     path_props = dict(
         path=path,
@@ -523,7 +559,7 @@ class AnnotatePaths(Interface):
             if requested_paths and refds_path:
                 for r in requested_paths:
                     p = r['path'] if isinstance(r, dict) else r
-                    p = resolve_path(p, ds=refds_path)
+                    p = _resolve_path(p, ds=refds_path)
                     if path_startswith(p, refds_path):
                         # all good
                         continue
@@ -563,6 +599,16 @@ class AnnotatePaths(Interface):
             # re-append the preserved paths:
             requested_paths = chain(requested_paths, iter(preserved_paths))
 
+        # Possibly to be used "cache" of known subdatasets per each parent
+        # to avoid re-querying subdatasets per each path.  The assumption here
+        # is that the list of sub-datasets for a given parent should not change
+        # through the execution of this loop, which (hypothetically) could be
+        # incorrect while annotating paths for some commands.
+        # TODO: verify this assumption and possibly add an argument to turn
+        #  caching off if/when needed, or provide some other way to invalidate
+        #  it
+        subdss_cache = {}
+
         # do not loop over unique(), this could be a list of dicts
         # we avoid duplicates manually below via `reported_paths`
         for path in requested_paths:
@@ -669,9 +715,15 @@ class AnnotatePaths(Interface):
                 # a dataset (without this info) -> record whether this is a known subdataset
                 # to its parent
                 containing_ds = Dataset(parent)
-                subdss = containing_ds.subdatasets(
-                    fulfilled=None, recursive=False,
-                    result_xfm=None, result_filter=None, return_type='list')
+                # Possibly "cache" the list of known subdss for parents we
+                # have encountered so far
+                if parent in subdss_cache:
+                    subdss = subdss_cache[parent]
+                else:
+                    subdss = containing_ds.subdatasets(
+                        fulfilled=None, recursive=False,
+                        result_xfm=None, result_filter=None, return_type='list')
+                    subdss_cache[parent] = subdss
                 if path in [s['path'] for s in subdss]:
                     if path_type == 'directory' or not lexists(path):
                         # first record that it isn't here, if just a dir or not here at all
