@@ -12,21 +12,28 @@ __docformat__ = 'restructuredtext'
 
 from os.path import join as opj
 from glob import glob
+import itertools
 from .base import Interface
 from ..utils import rmtree
 from ..support.param import Parameter
-from ..consts import ARCHIVES_TEMP_DIR
-from ..consts import ANNEX_TEMP_DIR
-from ..consts import SEARCH_INDEX_DOTGITDIR
+from ..consts import (
+    ARCHIVES_TEMP_DIR,
+    ANNEX_TEMP_DIR,
+    ANNEX_TRANSFER_DIR,
+    SEARCH_INDEX_DOTGITDIR,
+)
 
 from datalad.support.gitrepo import GitRepo
 from datalad.support.constraints import EnsureNone
-from datalad.distribution.dataset import EnsureDataset
-from datalad.distribution.dataset import require_dataset
-from datalad.distribution.dataset import datasetmethod
-from datalad.interface.annotate_paths import AnnotatePaths
-from datalad.interface.common_opts import recursion_flag
-from datalad.interface.common_opts import recursion_limit
+from datalad.distribution.dataset import (
+    EnsureDataset,
+    require_dataset,
+    datasetmethod,
+)
+from datalad.interface.common_opts import (
+    recursion_flag,
+    recursion_limit,
+)
 from datalad.interface.results import get_status_dict
 from datalad.interface.utils import eval_results
 from datalad.interface.base import build_doc
@@ -34,6 +41,9 @@ from datalad.interface.base import build_doc
 from logging import getLogger
 lgr = getLogger('datalad.api.clean')
 
+# needed API commands
+import datalad.distribution.subdatasets
+
 
 @build_doc
 class Clean(Interface):
@@ -58,7 +68,7 @@ class Clean(Interface):
         what=Parameter(
             args=("--what",),
             dest='what',
-            choices=('cached-archives', 'annex-tmp', 'search-index'),
+            choices=('cached-archives', 'annex-tmp', 'annex-transfer', 'search-index'),
             nargs="*",
             doc="""What to clean.  If none specified -- all known targets are
             cleaned"""),
@@ -72,32 +82,26 @@ class Clean(Interface):
     def __call__(dataset=None, what=None, recursive=False, recursion_limit=None):
         ds = require_dataset(dataset, purpose='clean-up')
         res_kwargs = dict(action='clean', logger=lgr, refds=ds.path)
-        for ap in AnnotatePaths.__call__(
-                dataset=ds.path,
+        for wds in itertools.chain([ds], ds.subdatasets(
+                fulfilled=True,
                 recursive=recursive,
                 recursion_limit=recursion_limit,
-                action='clean',
-                unavailable_path_status='impossible',
-                nondataset_path_status='impossible',
                 return_type='generator',
-                on_failure='ignore'):
-            if ap.get('status', None):
-                yield ap
-                continue
-            if ap.get('type', None) != 'dataset':
-                ap.update(status='impossible',
-                          message='only datasets can be cleaned')
-                yield ap
-                continue
-            d = ap['path']
+                result_renderer='disabled',
+                result_xfm='datasets') if recursive else []):
+            d = wds.path
             gitdir = GitRepo.get_git_dir(d)
+            DIRS_PLURAL = ("directory", "directories")
+            FILES_PLURAL = ("file", "files")
             for dirpath, flag, msg, sing_pl in [
                 (ARCHIVES_TEMP_DIR, "cached-archives",
-                 "temporary archive", ("directory", "directories")),
+                 "temporary archive", DIRS_PLURAL),
                 (ANNEX_TEMP_DIR, "annex-tmp",
-                 "temporary annex", ("file", "files")),
+                 "temporary annex", FILES_PLURAL),
+                (ANNEX_TRANSFER_DIR, "annex-transfer",
+                 "annex temporary transfer", DIRS_PLURAL),
                 (opj(gitdir, SEARCH_INDEX_DOTGITDIR), 'search-index',
-                 "metadata search index", ("file", "files")),
+                 "metadata search index", FILES_PLURAL),
             ]:
                 topdir = opj(d, dirpath)
                 lgr.debug("Considering to clean %s:%s", d, dirpath)
