@@ -10,13 +10,16 @@
 
 """
 
-from datalad.tests.utils import known_failure_direct_mode
+import logging
 
 from copy import deepcopy
 
+import os
 from os.path import join as opj
 from os.path import basename
 from os.path import lexists
+from os.path import normpath
+from os.path import abspath
 
 from datalad.tests.utils import with_tree
 from datalad.tests.utils import with_tempfile
@@ -27,13 +30,20 @@ from datalad.tests.utils import assert_raises
 from datalad.tests.utils import assert_not_in
 from datalad.tests.utils import create_tree
 from datalad.tests.utils import slow
+from datalad.tests.utils import swallow_logs
+from datalad.tests.utils import known_failure_githubci_win
+from datalad.tests.utils import assert_cwd_unchanged
 
 
 from datalad.distribution.dataset import Dataset
 from datalad.api import annotate_paths
 from datalad.api import install
 from datalad.interface.annotate_paths import get_modified_subpaths
+from datalad.interface.annotate_paths import _resolve_path
 from datalad.utils import chpwd
+from datalad.utils import getpwd
+
+from datalad.interface.tests.test_utils import make_demo_hierarchy_datasets
 
 
 __docformat__ = 'restructuredtext'
@@ -53,17 +63,6 @@ demo_hierarchy = {
 }
 
 
-def make_demo_hierarchy_datasets(path, tree, parent=None):
-    if parent is None:
-        parent = Dataset(path).create(force=True)
-    for node, items in tree.items():
-        if isinstance(items, dict):
-            node_path = opj(path, node)
-            nodeds = Dataset(node_path).create(force=True)
-            make_demo_hierarchy_datasets(node_path, items, parent=nodeds)
-    return parent
-
-
 @with_tempfile(mkdir=True)
 def test_invalid_call(path):
     # inter-option dependencies
@@ -83,7 +82,7 @@ def test_invalid_call(path):
 def test_annotate_paths(dspath, nodspath):
     # this test doesn't use API`remove` to avoid circularities
     ds = make_demo_hierarchy_datasets(dspath, demo_hierarchy)
-    ds.add('.', recursive=True)
+    ds.save(recursive=True)
     ok_clean_git(ds.path)
 
     with chpwd(dspath):
@@ -105,6 +104,13 @@ def test_annotate_paths(dspath, nodspath):
             [{k: v for k, v in ap.items()
               if k not in ('registered_subds', 'raw_input', 'orig_request', 'refds')}
              for ap in annotate_paths(dataset='b', recursive=True)])
+
+        # when we point to a list of directories, there should be no
+        # multiple rediscoveries of the subdatasets
+        with swallow_logs(new_level=logging.DEBUG) as cml:
+            annotate_paths(path=['a', 'b'])
+            eq_(cml.out.count('Resolved dataset for subdataset reporting/modification'), 1)
+
     # now do it again, pointing to the ds directly
     res = ds.annotate_paths(on_failure='ignore')
     # no request, no refds, but otherwise the same
@@ -222,18 +228,18 @@ def test_annotate_paths(dspath, nodspath):
     eq_(orig_res, res_recursion_again)
 
 
+@known_failure_githubci_win
 @slow  # 11.0891s
 @with_tree(demo_hierarchy['b'])
-@known_failure_direct_mode  #FIXME
 def test_get_modified_subpaths(path):
     ds = Dataset(path).create(force=True)
     suba = ds.create('ba', force=True)
     subb = ds.create('bb', force=True)
     subsub = ds.create(opj('bb', 'bba', 'bbaa'), force=True)
-    ds.add('.', recursive=True)
+    ds.save(recursive=True)
     ok_clean_git(path)
 
-    orig_base_commit = ds.repo.repo.commit().hexsha
+    orig_base_commit = ds.repo.get_hexsha()
 
     # nothing was modified compared to the status quo, output must be empty
     eq_([],
@@ -243,7 +249,7 @@ def test_get_modified_subpaths(path):
 
     # modify one subdataset
     create_tree(subsub.path, {'added': 'test'})
-    subsub.add('added')
+    subsub.save('added')
 
     # it will replace the requested path with the path of the closest
     # submodule that is modified
@@ -285,7 +291,7 @@ def test_get_modified_subpaths(path):
         type='dataset', path=suba.path)
 
     # add/save everything, become clean
-    ds.add('.', recursive=True)
+    ds.save(recursive=True)
     ok_clean_git(path)
     # nothing is reported as modified
     assert_result_count(
@@ -330,17 +336,16 @@ def test_get_modified_subpaths(path):
 @slow  # 41.5367s
 @with_tree(demo_hierarchy)
 @with_tempfile(mkdir=True)
-@known_failure_direct_mode  #FIXME
 def test_recurseinto(dspath, dest):
     # make fresh dataset hierarchy
     ds = make_demo_hierarchy_datasets(dspath, demo_hierarchy)
-    ds.add('.', recursive=True)
+    ds.save(recursive=True)
     # label intermediate dataset as 'norecurseinto'
     res = Dataset(opj(ds.path, 'b')).subdatasets(
         contains='bb',
         set_property=[('datalad-recursiveinstall', 'skip')])
     assert_result_count(res, 1, path=opj(ds.path, 'b', 'bb'))
-    ds.add('b/', recursive=True)
+    ds.save('b', recursive=True)
     ok_clean_git(ds.path)
 
     # recursive install, should skip the entire bb branch
@@ -366,8 +371,34 @@ def test_recurseinto(dspath, dest):
     # explicitly -- must get it installed
     dest = install(source=ds.path, path=dest)
     res = dest.get(['.', opj('b', 'bb')], get_data=False, recursive=True)
-    assert_result_count(res, 8)
-    assert_result_count(res, 8, type='dataset')
+    assert_result_count(res, 7)
+    assert_result_count(res, 7, type='dataset')
     assert_result_count(res, 1, type='dataset',
                         path=opj(dest.path, 'b', 'bb'))
     assert(Dataset(opj(dest.path, 'b', 'bb')).is_installed())
+
+
+@assert_cwd_unchanged
+@with_tempfile(mkdir=True)
+def test_resolve_path(somedir):
+
+    abs_path = abspath(somedir)  # just to be sure
+    rel_path = "some"
+    expl_path_cur = opj(os.curdir, rel_path)
+    expl_path_par = opj(os.pardir, rel_path)
+
+    eq_(_resolve_path(abs_path), abs_path)
+
+    current = getpwd()
+    # no Dataset => resolve using cwd:
+    eq_(_resolve_path(abs_path), abs_path)
+    eq_(_resolve_path(rel_path), opj(current, rel_path))
+    eq_(_resolve_path(expl_path_cur), normpath(opj(current, expl_path_cur)))
+    eq_(_resolve_path(expl_path_par), normpath(opj(current, expl_path_par)))
+
+    # now use a Dataset as reference:
+    ds = Dataset(abs_path)
+    eq_(_resolve_path(abs_path, ds), abs_path)
+    eq_(_resolve_path(rel_path, ds), opj(abs_path, rel_path))
+    eq_(_resolve_path(expl_path_cur, ds), normpath(opj(current, expl_path_cur)))
+    eq_(_resolve_path(expl_path_par, ds), normpath(opj(current, expl_path_par)))
