@@ -12,7 +12,6 @@
 
 __docformat__ = 'restructuredtext'
 
-from datalad.tests.utils import known_failure_direct_mode
 
 
 from os.path import join as opj
@@ -33,6 +32,7 @@ from datalad.tests.utils import ok_
 from datalad.tests.utils import eq_
 from datalad.tests.utils import assert_status
 from datalad.tests.utils import assert_result_count
+from datalad.tests.utils import known_failure_githubci_win
 
 
 @known_failure_windows
@@ -46,67 +46,58 @@ def test_magic_number():
     eq_(out.strip(), PRE_INIT_COMMIT_SHA)
 
 
+@known_failure_githubci_win
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
 def test_diff(path, norepo):
-    with chpwd(norepo):
-        assert_status('impossible', diff(on_failure='ignore'))
     ds = Dataset(path).create()
     ok_clean_git(ds.path)
     # reports stupid revision input
     assert_result_count(
-        ds.diff(revision='WTF', on_failure='ignore'),
+        ds._diff(revision='WTF', on_failure='ignore'),
         1,
         status='impossible',
         message="fatal: bad revision 'WTF'")
-    assert_result_count(ds.diff(), 0)
+    assert_result_count(ds._diff(), 0)
     # no diff
-    assert_result_count(ds.diff(), 0)
-    assert_result_count(ds.diff(revision='HEAD'), 0)
+    assert_result_count(ds._diff(), 0)
+    assert_result_count(ds._diff(revision='HEAD'), 0)
     # bogus path makes no difference
-    assert_result_count(ds.diff(path='THIS', revision='HEAD'), 0)
-    # comparing to a previous state we should get a diff in most cases
-    # for this test, let's not care what exactly it is -- will do later
-    assert len(ds.diff(revision='HEAD~1')) > 0
+    assert_result_count(ds._diff(path='THIS', revision='HEAD'), 0)
     # let's introduce a known change
     create_tree(ds.path, {'new': 'empty'})
-    ds.add('.', to_git=True)
+    ds.save(to_git=True)
     ok_clean_git(ds.path)
-    res = ds.diff(revision='HEAD~1')
+    res = ds._diff(revision='HEAD~1')
     assert_result_count(res, 1)
     assert_result_count(
         res, 1, action='diff', path=opj(ds.path, 'new'), state='added')
-    # we can also find the diff without going through the dataset explicitly
-    with chpwd(ds.path):
-        assert_result_count(
-            diff(revision='HEAD~1'), 1,
-            action='diff', path=opj(ds.path, 'new'), state='added')
     # no diff against HEAD
-    assert_result_count(ds.diff(), 0)
+    assert_result_count(ds._diff(), 0)
     # modify known file
     create_tree(ds.path, {'new': 'notempty'})
     for diffy in (None, 'HEAD'):
-        res = ds.diff(revision=diffy)
+        res = ds._diff(revision=diffy)
         assert_result_count(res, 1)
         assert_result_count(
             res, 1, action='diff', path=opj(ds.path, 'new'), state='modified')
     # but if we give another path, it doesn't show up
-    assert_result_count(ds.diff('otherpath'), 0)
+    assert_result_count(ds._diff('otherpath'), 0)
     # giving the right path must work though
     assert_result_count(
-        ds.diff('new'), 1,
+        ds._diff('new'), 1,
         action='diff', path=opj(ds.path, 'new'), state='modified')
     # stage changes
-    ds.add('.', to_git=True, save=False)
+    ds.repo.add('.', git=True)
     # no diff, because we staged the modification
-    assert_result_count(ds.diff(), 0)
+    assert_result_count(ds._diff(), 0)
     # but we can get at it
     assert_result_count(
-        ds.diff(staged=True), 1,
+        ds._diff(staged=True), 1,
         action='diff', path=opj(ds.path, 'new'), state='modified')
     # OR
     assert_result_count(
-        ds.diff(revision='HEAD'), 1,
+        ds._diff(revision='HEAD'), 1,
         action='diff', path=opj(ds.path, 'new'), state='modified')
     ds.save()
     ok_clean_git(ds.path)
@@ -115,46 +106,46 @@ def test_diff(path, norepo):
     create_tree(ds.path, {'deep': {'down': 'untracked', 'down2': 'tobeadded'}})
     # a plain diff should report the untracked file
     # but not directly, because the parent dir is already unknown
-    res = ds.diff()
+    res = ds._diff()
     assert_result_count(res, 1)
     assert_result_count(
         res, 1, state='untracked', type='directory', path=opj(ds.path, 'deep'))
     # report of individual files is also possible
     assert_result_count(
-        ds.diff(report_untracked='all'), 2, state='untracked', type='file')
+        ds._diff(report_untracked='all'), 2, state='untracked', type='file')
     # an unmatching path will hide this result
-    assert_result_count(ds.diff(path='somewhere'), 0)
+    assert_result_count(ds._diff(path='somewhere'), 0)
     # perfect match and anything underneath will do
     assert_result_count(
-        ds.diff(path='deep'), 1, state='untracked', path=opj(ds.path, 'deep'),
+        ds._diff(path='deep'), 1, state='untracked', path=opj(ds.path, 'deep'),
         type='directory')
     assert_result_count(
-        ds.diff(path='deep'), 1,
+        ds._diff(path='deep'), 1,
         state='untracked', path=opj(ds.path, 'deep'))
     # now we stage on of the two files in deep
-    ds.add(opj('deep', 'down2'), to_git=True, save=False)
+    ds.repo.add(opj('deep', 'down2'), git=True)
     # without any reference it will ignore the staged stuff and report the remaining
     # untracked file
     assert_result_count(
-        ds.diff(), 1, state='untracked', path=opj(ds.path, 'deep', 'down'),
+        ds._diff(), 1, state='untracked', path=opj(ds.path, 'deep', 'down'),
         type='file')
-    res = ds.diff(staged=True)
+    res = ds._diff(staged=True)
     assert_result_count(
         res, 1, state='untracked', path=opj(ds.path, 'deep', 'down'), type='file')
     assert_result_count(
         res, 1, state='added', path=opj(ds.path, 'deep', 'down2'), type='file')
 
 
+@known_failure_githubci_win
 @with_tempfile(mkdir=True)
-@known_failure_direct_mode  #FIXME
 def test_diff_recursive(path):
     ds = Dataset(path).create()
     sub = ds.create('sub')
     # look at the last change, and confirm a dataset was added
-    res = ds.diff(revision='HEAD~1..HEAD')
+    res = ds._diff(revision='HEAD~1..HEAD')
     assert_result_count(res, 1, action='diff', state='added', path=sub.path, type='dataset')
     # now recursive
-    res = ds.diff(recursive=True, revision='HEAD~1..HEAD')
+    res = ds._diff(recursive=True, revision='HEAD~1..HEAD')
     # we also get the entire diff of the subdataset from scratch
     assert_status('ok', res)
     ok_(len(res) > 3)
@@ -163,32 +154,32 @@ def test_diff_recursive(path):
 
     # now we add a file to just the parent
     create_tree(ds.path, {'onefile': 'tobeadded', 'sub': {'twofile': 'tobeadded'}})
-    res = ds.diff(recursive=True, report_untracked='all')
+    res = ds._diff(recursive=True, report_untracked='all')
     assert_result_count(res, 3)
     assert_result_count(res, 1, action='diff', state='untracked', path=opj(ds.path, 'onefile'), type='file')
     assert_result_count(res, 1, action='diff', state='modified', path=sub.path, type='dataset')
     assert_result_count(res, 1, action='diff', state='untracked', path=opj(sub.path, 'twofile'), type='file')
     # save sub
-    sub.add('.')
+    sub.save()
     # save sub in parent
-    ds.save()
+    ds.save(sub.path)
     # save addition in parent
-    ds.add('.')
+    ds.save()
     ok_clean_git(ds.path)
     # look at the last change, only one file was added
-    res = ds.diff(revision='HEAD~1..HEAD')
+    res = ds._diff(revision='HEAD~1..HEAD')
     assert_result_count(res, 1)
     assert_result_count(res, 1, action='diff', state='added', path=opj(ds.path, 'onefile'), type='file')
 
     # now the exact same thing with recursion, must not be different from the call
     # above
-    res = ds.diff(recursive=True, revision='HEAD~1..HEAD')
+    res = ds._diff(recursive=True, revision='HEAD~1..HEAD')
     assert_result_count(res, 1)
     # last change in parent
     assert_result_count(res, 1, action='diff', state='added', path=opj(ds.path, 'onefile'), type='file')
 
     # one further back brings in the modified subdataset, and the added file within it
-    res = ds.diff(recursive=True, revision='HEAD~2..HEAD')
+    res = ds._diff(recursive=True, revision='HEAD~2..HEAD')
     assert_result_count(res, 3)
     assert_result_count(res, 1, action='diff', state='added', path=opj(ds.path, 'onefile'), type='file')
     assert_result_count(res, 1, action='diff', state='added', path=opj(sub.path, 'twofile'), type='file')
@@ -203,7 +194,6 @@ def test_diff_recursive(path):
     'modified': 'original_content',
     'untracked': 'dirt',
 })
-@known_failure_direct_mode  #FIXME
 def test_diff_helper(path):
     # make test dataset components of interesting states
     ds = Dataset.create(path, force=True)
@@ -213,10 +203,10 @@ def test_diff_helper(path):
     sub_clean = ds.create('sub_clean', force=True)
     # proper submodule, but commited modifications not commited in parent
     sub_modified = ds.create('sub_modified', force=True)
-    sub_modified.add('modified')
+    sub_modified.save('modified')
     # proper submodule with untracked changes
     sub_dirty = ds.create('sub_dirty', force=True)
-    ds.add(['clean', 'modified'])
+    ds.save(['clean', 'modified'])
     ds.unlock('modified')
     with open(opj(ds.path, 'modified'), 'w') as f:
         f.write('modified_content')
