@@ -9,16 +9,20 @@
 """Create and update a dataset from a list of URLs.
 """
 
-from collections import Mapping
+try:
+    from collections.abc import Mapping
+except ImportError:  # Python <= 3.3
+    from collections import Mapping
+
 from functools import partial
 import logging
 import os
 import re
 import string
 
-from six import string_types
-from six.moves.urllib.parse import urlparse
+from urllib.parse import urlparse
 
+from datalad.distribution.dataset import resolve_path
 from datalad.dochelpers import exc_str
 from datalad.log import log_progress, with_result_progress
 from datalad.interface.base import Interface
@@ -31,6 +35,7 @@ from datalad.support.path import split_ext
 from datalad.support.s3 import get_versioned_url
 from datalad.utils import (
     assure_list,
+    get_suggestions_msg,
     unlink,
 )
 
@@ -86,7 +91,7 @@ class Formatter(string.Formatter):
             return super(Formatter, self).get_value(
                 key, args, kwargs)
 
-        if self.missing is not None and isinstance(value, string_types):
+        if self.missing is not None and isinstance(value, str):
             return value or self.missing
         return value
 
@@ -252,14 +257,22 @@ def _read(stream, input_type):
     if input_type == "csv":
         import csv
         csvrows = csv.reader(stream)
-        headers = next(csvrows)
+        try:
+            headers = next(csvrows)
+        except StopIteration:
+            raise ValueError("Failed to read CSV rows from {}".format(stream))
         lgr.debug("Taking %s fields from first line as headers: %s",
                   len(headers), headers)
         idx_map = dict(enumerate(headers))
         rows = [dict(zip(headers, r)) for r in csvrows]
     elif input_type == "json":
         import json
-        rows = json.load(stream)
+        try:
+            rows = json.load(stream)
+        except json.decoder.JSONDecodeError as e:
+            raise ValueError(
+                "Failed to read JSON from stream {}: {}"
+                .format(stream, exc_str(e)))
         # For json input, we do not support indexing by position,
         # only names.
         idx_map = {}
@@ -268,10 +281,29 @@ def _read(stream, input_type):
     return rows, idx_map
 
 
+def _get_placeholder_exception(exc, msg_prefix, known):
+    """Recast KeyError as a ValueError with close-match suggestions.
+    """
+    value = exc.args[0]
+    if isinstance(value, str):
+        sugmsg = get_suggestions_msg(value, known)
+    else:
+        sugmsg = "Out-of-bounds or unsupported index."
+    # Note: Keeping this a KeyError is probably more appropriate but then the
+    # entire message, which KeyError takes as the key, will be rendered with
+    # outer quotes.
+    return ValueError("{}: {}{}{}"
+                      .format(msg_prefix, exc, ". " if sugmsg else "", sugmsg))
+
+
 def _format_filenames(format_fn, rows, row_infos):
     subpaths = set()
     for row, info in zip(rows, row_infos):
-        filename = format_fn(row)
+        try:
+            filename = format_fn(row)
+        except KeyError as exc:
+            raise _get_placeholder_exception(
+                exc, "Unknown placeholder in file name", row)
         filename, spaths = get_subpaths(filename)
         subpaths |= set(spaths)
         info["filename"] = filename
@@ -407,12 +439,15 @@ def extract(stream, input_type, url_format="{0}", filename_format="{1}",
     Returns
     -------
     A tuple where the first item is a list with a dict of extracted information
-    for each row in `stream` and the second item is a set that contains all the
-    subdataset paths.
+    for each row in `stream` and the second item a list subdataset paths,
+    sorted breadth-first.
     """
     meta = assure_list(meta)
 
     rows, colidx_to_name = _read(stream, input_type)
+    if not rows:
+        lgr.warning("No rows found in %s", stream)
+        return [], []
 
     fmt = Formatter(colidx_to_name, missing_value)  # For URL and meta
     format_url = partial(fmt.format, url_format)
@@ -436,7 +471,11 @@ def extract(stream, input_type, url_format="{0}", filename_format="{1}",
     rows_with_url = []
     infos = []
     for row in rows:
-        url = format_url(row)
+        try:
+            url = format_url(row)
+        except KeyError as exc:
+            raise _get_placeholder_exception(
+                exc, "Unknown placeholder in URL", row)
         if not url or url == missing_value:
             continue  # pragma: no cover, peephole optimization
         rows_with_url.append(row)
@@ -507,14 +546,27 @@ def add_urls(rows, ifexists=None, options=None):
 def add_meta(rows):
     """Call `git annex metadata --set` using information in `rows`.
     """
-    from mock import patch
+    from unittest.mock import patch
 
     for row in rows:
         ds, filename = row["ds"], row["ds_filename"]
-
         with patch.object(ds.repo, "always_commit", False):
+            res = ds.repo.add(filename)
+            res_status = 'notneeded' if not res \
+                else 'ok' if res.get('success', False) \
+                else 'error'
+
+            yield dict(
+                action='add',
+                # decorator dies with Path()
+                path=str(ds.pathobj / filename),
+                type='file',
+                status=res_status,
+                parentds=ds.path,
+            )
+
             lgr.debug("Adding metadata to %s in %s", filename, ds.path)
-            for a in ds.repo.set_metadata(filename, add=row["meta_args"]):
+            for a in ds.repo.set_metadata_(filename, add=row["meta_args"]):
                 res = annexjson2result(a, ds, type="file", logger=lgr)
                 # Don't show all added metadata for the file because that
                 # could quickly flood the output.
@@ -629,10 +681,13 @@ class Addurls(Interface):
             args=("filenameformat",),
             metavar="FILENAME-FORMAT",
             doc="""Like `URL-FORMAT`, but this format string specifies the file
-            to which the URL's content will be downloaded.  The file name may
-            contain directories.  The separator "//" can be used to indicate
-            that the left-side directory should be created as a new subdataset.
-            See the 'Format Specification' section above."""),
+            to which the URL's content will be downloaded. The name should be a
+            relative path and will be taken as relative to the top-level
+            dataset, regardless of whether it is specified via [PY: `dataset`
+            PY][CMD: --dataset CMD]) or inferred. The file name may contain
+            directories. The separator "//" can be used to indicate that the
+            left-side directory should be created as a new subdataset. See the
+            'Format Specification' section above."""),
         input_type=Parameter(
             args=("-t", "--input-type"),
             metavar="TYPE",
@@ -676,13 +731,12 @@ class Addurls(Interface):
             Underneath, this passes the --fast flag to `git annex addurl`."""),
         ifexists=Parameter(
             args=("--ifexists",),
-            metavar="ACTION",
             doc="""What to do if a constructed file name already exists.  The
             default behavior is to proceed with the `git annex addurl`, which
             will fail if the file size has changed.  If set to 'overwrite',
             remove the old file before adding the new one.  If set to 'skip',
             do not add the new file.""",
-            constraints=EnsureNone() | EnsureChoice("overwrite", "skip")),
+            constraints=EnsureChoice(None, "overwrite", "skip")),
         missing_value=Parameter(
             args=("--missing-value",),
             metavar="VALUE",
@@ -694,7 +748,15 @@ class Addurls(Interface):
             args=("--version-urls",),
             action="store_true",
             doc="""Try to add a version ID to the URL. This currently only has
-            an effect on URLs for AWS S3 buckets."""),
+            an effect on HTTP URLs for AWS S3 buckets. s3:// URL versioning is
+            not yet supported, but any URL that already contains a "versionId="
+            parameter will be used as is."""),
+        cfg_proc=Parameter(
+            args=("-c", "--cfg-proc"),
+            metavar="PROC",
+            action='append',
+            doc="""Pass this [PY: cfg_proc PY][CMD: --cfg_proc CMD] value when
+            calling `create` to make datasets."""),
     )
 
     @staticmethod
@@ -703,7 +765,8 @@ class Addurls(Interface):
     def __call__(dataset, urlfile, urlformat, filenameformat,
                  input_type="ext", exclude_autometa=None, meta=None,
                  message=None, dry_run=False, fast=False, ifexists=None,
-                 missing_value=None, save=True, version_urls=False):
+                 missing_value=None, save=True, version_urls=False,
+                 cfg_proc=None):
         # Temporarily work around gh-2269.
         url_file = urlfile
         url_format, filename_format = urlformat, filenameformat
@@ -716,14 +779,16 @@ class Addurls(Interface):
 
         lgr = logging.getLogger("datalad.plugin.addurls")
 
-        dataset = require_dataset(dataset, check_installed=False)
-        if dataset.repo and not isinstance(dataset.repo, AnnexRepo):
+        ds = require_dataset(dataset, check_installed=False)
+        if ds.repo and not isinstance(ds.repo, AnnexRepo):
             yield get_status_dict(action="addurls",
-                                  ds=dataset,
+                                  ds=ds,
                                   status="error",
                                   message="not an annex repo")
             return
 
+        url_file = str(resolve_path(url_file, dataset))
+
         if input_type == "ext":
             extension = os.path.splitext(url_file)[1]
             input_type = "json" if extension == ".json" else "csv"
@@ -737,14 +802,21 @@ class Addurls(Interface):
                                          missing_value)
             except (ValueError, RequestException) as exc:
                 yield get_status_dict(action="addurls",
-                                      ds=dataset,
+                                      ds=ds,
                                       status="error",
                                       message=exc_str(exc))
                 return
 
+        if not rows:
+            yield get_status_dict(action="addurls",
+                                  ds=ds,
+                                  status="notneeded",
+                                  message="No rows to process")
+            return
+
         if len(rows) != len(set(row["filename"] for row in rows)):
             yield get_status_dict(action="addurls",
-                                  ds=dataset,
+                                  ds=ds,
                                   status="error",
                                   message=("There are file name collisions; "
                                            "consider using {_repindex}"))
@@ -756,44 +828,46 @@ class Addurls(Interface):
             for row in rows:
                 lgr.info("Would download %s to %s",
                          row["url"],
-                         os.path.join(dataset.path, row["filename"]))
+                         os.path.join(ds.path, row["filename"]))
                 lgr.info("Metadata: %s",
                          sorted(u"{}={}".format(k, v)
                                 for k, v in row["meta_args"].items()))
             yield get_status_dict(action="addurls",
-                                  ds=dataset,
+                                  ds=ds,
                                   status="ok",
                                   message="dry-run finished")
             return
 
-        if not dataset.repo:
+        if not ds.repo:
             # Populate a new dataset with the URLs.
-            for r in dataset.create(result_xfm=None, return_type='generator',
-                                    save=save):
+            for r in ds.create(result_xfm=None,
+                               return_type='generator',
+                               cfg_proc=cfg_proc):
                 yield r
 
         annex_options = ["--fast"] if fast else []
 
         for spath in subpaths:
-            if os.path.exists(os.path.join(dataset.path, spath)):
+            if os.path.exists(os.path.join(ds.path, spath)):
                 lgr.warning(
                     "Not creating subdataset at existing path: %s",
                     spath)
             else:
-                for r in dataset.create(spath, result_xfm=None,
-                                        return_type='generator', save=save):
+                for r in ds.create(spath, result_xfm=None,
+                                   cfg_proc=cfg_proc,
+                                   return_type='generator'):
                     yield r
 
         for row in rows:
             # Add additional information that we'll need for various
             # operations.
-            filename_abs = os.path.join(dataset.path, row["filename"])
+            filename_abs = os.path.join(ds.path, row["filename"])
             if row["subpath"]:
-                ds_current = Dataset(os.path.join(dataset.path,
+                ds_current = Dataset(os.path.join(ds.path,
                                                   row["subpath"]))
                 ds_filename = os.path.relpath(filename_abs, ds_current.path)
             else:
-                ds_current = dataset
+                ds_current = ds
                 ds_filename = row["filename"]
             row.update({"filename_abs": filename_abs,
                         "ds": ds_current,
@@ -835,17 +909,12 @@ url_format='{}'
 filename_format='{}'""".format(url_file, url_format, filename_format)
 
         if files_to_add:
-            for r in dataset.add(files_to_add, save=False):
-                yield r
-
             meta_rows = [r for r in rows if r["filename_abs"] in files_to_add]
             for r in add_meta(meta_rows):
                 yield r
 
-            # Save here rather than the add call above to trigger a metadata
-            # commit on the git-annex branch.
             if save:
-                for r in dataset.save(message=msg, recursive=True):
+                for r in ds.save(path=files_to_add, message=msg, recursive=True):
                     yield r
 
 
