@@ -45,7 +45,7 @@ def test_failure(path):
 @with_tree(_dataset_template)
 def test_archive(path):
     ds = Dataset(opj(path, 'ds')).create(force=True)
-    ds.add('.')
+    ds.save()
     committed_date = ds.repo.get_commit_date()
     default_outname = opj(path, 'datalad_{}.tar.gz'.format(ds.id))
     with chpwd(path):
@@ -87,12 +87,6 @@ def test_archive(path):
     check_contents(custom_outname, 'myexport')
 
     # now loose some content
-    if ds.repo.is_direct_mode():
-        # in direct mode the add() aove commited directly to the annex/direct/master
-        # branch, hence drop will have no effect (notneeded)
-        # this might be undesired behavior (or not), but this is not the place to test
-        # for it
-        return
     ds.drop('file_up', check=False)
     assert_raises(IOError, ds.export_archive, filename=opj(path, 'my'))
     ds.export_archive(filename=opj(path, 'partial'), missing_content='ignore')
@@ -102,7 +96,7 @@ def test_archive(path):
 @with_tree(_dataset_template)
 def test_zip_archive(path):
     ds = Dataset(opj(path, 'ds')).create(force=True, no_annex=True)
-    ds.add('.')
+    ds.save()
     with chpwd(path):
         ds.export_archive(filename='my', archivetype='zip')
         assert_true(os.path.exists('my.zip'))
@@ -114,4 +108,4 @@ def test_zip_archive(path):
     # should be able to export without us cd'ing to that ds directory
     ds.export_archive(filename=ds.path, archivetype='zip')
     default_name = 'datalad_{}.zip'.format(ds.id)
-    assert_true(os.path.exists(os.path.join(ds.path, default_name)))
\ No newline at end of file
+    assert_true(os.path.exists(os.path.join(ds.path, default_name)))
