@@ -11,115 +11,62 @@
 """
 
 import hashlib
-import patoolib
-from .external_versions import external_versions
-# There were issues, so let's stay consistently with recent version
-assert(external_versions["patoolib"] >= "1.7")
-
 import os
 import tempfile
-from .path import join as opj, exists, abspath, isabs, normpath, relpath, pardir, isdir
-from .path import sep as opsep
-from .path import realpath
-from six import next, PY2
-from six.moves.urllib.parse import unquote as urlunquote
-
+from urllib.parse import unquote as urlunquote
 import string
 import random
+import logging
 
-from .locking import lock_if_check_fails
-from ..utils import (
-    any_re_search,
-    assure_bytes,
-    chpwd,
-    rmdir,
-    unlink,
+from datalad.support.path import (
+    join as opj,
+    exists,
+    abspath,
+    isabs,
+    normpath,
+    relpath,
+    pardir,
+    isdir,
+    realpath,
+    sep as opsep,
 )
 
-import logging
-lgr = logging.getLogger('datalad.files')
+from datalad.support.locking import lock_if_check_fails
+from datalad.support.external_versions import external_versions
+from datalad.consts import ARCHIVES_TEMP_DIR
+from datalad.utils import (
+    any_re_search,
+    ensure_bytes,
+    ensure_unicode,
+    unlink,
+    rmdir,
+    rmtemp,
+    rmtree,
+    get_tempfile_kwargs,
+    on_windows,
+)
+from datalad import cfg
+from datalad.config import anything2bool
 
-# Monkey-patch patoolib's logging, so it logs coherently with the rest of
-# datalad
-import patoolib.util
-#
-# Seems have managed with swallow_outputs
-#
-# def _patool_log(level, msg):
-#     lgr.log(level, "patool: %s" % msg)
-#
-# def _patool_log_info(msg, *args, **kwargs):
-#     _patool_log(logging.DEBUG, msg)
-#
-# def _patool_log_error(msg, *args, **kwargs):
-#     _patool_log(logging.ERROR, msg)
-#
-# patoolib.util.log_info = _patool_log_info
-# patoolib.util.log_error = _patool_log_error
-# patoolib.util.log_internal_error = _patool_log_error
+# fall back on patool, if a functional implementation is available
+# (i.e. not on windows), it is requested, or 7z is not found
+if not on_windows and (
+        cfg.obtain(
+            'datalad.runtime.use-patool', default=False,
+            valtype=anything2bool) or not external_versions['cmd:7z']):
+    from datalad.support.archive_utils_patool import (
+        decompress_file as _decompress_file,
+        # other code expects this to be here
+        compress_files
+    )
+else:
+    from datalad.support.archive_utils_7z import (
+        decompress_file as _decompress_file,
+        # other code expects this to be here
+        compress_files
+    )
 
-# we need to decorate patool.util.run
-# because otherwise it just lets processes to spit out everything to std and we
-# do want to use it at "verbosity>=0" so we could get idea on what is going on.
-# And I don't want to mock for every invocation
-from ..support.exceptions import CommandError
-from ..utils import swallow_outputs
-from ..utils import rmtemp
-from ..cmd import Runner
-from ..consts import ARCHIVES_TEMP_DIR
-from ..utils import rmtree
-from ..utils import get_tempfile_kwargs
-from ..utils import assure_unicode
-
-from ..utils import on_windows
-
-_runner = Runner()
-
-
-def _patool_run(cmd, verbosity=0, **kwargs):
-    """Decorated runner for patool so it doesn't spit out outputs to stdout"""
-    # use our runner
-    try:
-        # kwargs_ = kwargs[:];         kwargs_['shell'] = True
-        _runner.run(cmd, **kwargs)
-        return 0
-    except CommandError as e:
-        return e.code
-    except Exception as e:
-        lgr.error("While invoking runner caught unexpected exception: %s" % e)
-        return 100  # unknown beast
-patoolib.util.run = _patool_run
-
-
-# yoh: only keys are used atm, logic in decompress_file is replaced to use
-# patool
-
-DECOMPRESSORS = {
-    '\.(tar\.bz|tbz)$': 'tar -xjvf %(file)s -C %(dir)s',
-    '\.(tar\.xz)$': 'tar -xJvf %(file)s -C %(dir)s',
-    '\.(tar\.gz|tgz)$': 'tar -xzvf %(file)s -C %(dir)s',
-    '\.(zip)$': 'unzip %(file)s -d %(dir)s',
-}
-
-
-def unixify_path(path):
-    """On windows convert paths from drive:\d\file to /drive/d/file
-
-    This overcomes problems with various cmdline tools we are to use,
-    such as tar etc
-    """
-    if on_windows:
-        drive, path_ = os.path.splitdrive(path)
-        path_ = path_.split(os.sep)
-        path_ = '/'.join(path_)
-        if drive:
-            # last one must be :
-            assert(drive[-1] == ":")
-            return '/%s%s' % (drive[:-1], path_)
-        else:
-            return path_
-    else:
-        return path
+lgr = logging.getLogger('datalad.support.archives')
 
 
 def decompress_file(archive, dir_, leading_directories='strip'):
@@ -138,41 +85,7 @@ def decompress_file(archive, dir_, leading_directories='strip'):
         lgr.debug("Creating directory %s to extract archive into" % dir_)
         os.makedirs(dir_)
 
-    with swallow_outputs() as cmo:
-        archive = assure_bytes(archive)
-        dir_ = assure_bytes(dir_)
-        patoolib.util.check_existing_filename(archive)
-        patoolib.util.check_existing_filename(dir_, onlyfiles=False)
-        # Call protected one to avoid the checks on existence on unixified path
-        outdir = unixify_path(dir_)
-        if not PY2:
-            # should be supplied in PY3 to avoid b''
-            outdir = assure_unicode(outdir)
-            archive = assure_unicode(archive)
-        patoolib._extract_archive(unixify_path(archive),
-                                  outdir=outdir,
-                                  verbosity=100)
-        if cmo.out:
-            lgr.debug("patool gave stdout:\n%s" % cmo.out)
-        if cmo.err:
-            lgr.debug("patool gave stderr:\n%s" % cmo.err)
-
-    # Note: (ben) Experienced issue, where extracted tarball
-    # lacked execution bit of directories, leading to not being
-    # able to delete them while having write permission.
-    # Can't imagine a situation, where we would want to fail on
-    # that kind of mess. So, to be sure set it.
-
-    if not on_windows:
-        os.chmod(dir_,
-                 os.stat(dir_).st_mode |
-                 os.path.stat.S_IEXEC)
-        for root, dirs, files in os.walk(dir_, followlinks=False):
-            for d in dirs:
-                subdir = opj(root, d)
-                os.chmod(subdir,
-                         os.stat(subdir).st_mode |
-                         os.path.stat.S_IEXEC)
+    _decompress_file(archive, dir_)
 
     if leading_directories == 'strip':
         _, dirs, files = next(os.walk(dir_))
@@ -190,41 +103,13 @@ def decompress_file(archive, dir_, leading_directories='strip'):
         raise NotImplementedError("Not supported %s" % leading_directories)
 
 
-def compress_files(files, archive, path=None, overwrite=True):
-    """Compress `files` into an `archive` file
-
-    Parameters
-    ----------
-    files : list of str
-    archive : str
-    path : str
-      Alternative directory under which compressor will be invoked, to e.g.
-      take into account relative paths of files and/or archive
-    overwrite : bool
-      Whether to allow overwriting the target archive file if one already exists
-    """
-    with swallow_outputs() as cmo:
-        with chpwd(path):
-            if not overwrite:
-                patoolib.util.check_new_filename(archive)
-            patoolib.util.check_archive_filelist(files)
-            # Call protected one to avoid the checks on existence on unixified path
-            patoolib._create_archive(unixify_path(archive),
-                                     [unixify_path(f) for f in files],
-                                     verbosity=100)
-        if cmo.out:
-            lgr.debug("patool gave stdout:\n%s" % cmo.out)
-        if cmo.err:
-            lgr.debug("patool gave stderr:\n%s" % cmo.err)
-
-
 def _get_cached_filename(archive):
     """A helper to generate a filename which has original filename and additional suffix
     which wouldn't collide across files with the same name from different locations
     """
     #return "%s_%s" % (basename(archive), hashlib.md5(archive).hexdigest()[:5])
     # per se there is no reason to maintain any long original name here.
-    archive_cached = hashlib.md5(assure_bytes(realpath(archive))).hexdigest()[:10]
+    archive_cached = hashlib.md5(ensure_bytes(realpath(archive))).hexdigest()[:10]
     lgr.debug("Cached directory for archive %s is %s", archive, archive_cached)
     return archive_cached
 
@@ -267,7 +152,7 @@ class ArchivesCache(object):
             path = tempfile.mktemp(**get_tempfile_kwargs())
         self._path = path
         self.persistent = persistent
-        # TODO?  assure that it is absent or we should allow for it to persist a bit?
+        # TODO?  ensure that it is absent or we should allow for it to persist a bit?
         #if exists(path):
         #    self._clean_cache()
         self._archives = {}
@@ -279,7 +164,7 @@ class ArchivesCache(object):
                 self._made_path = True
                 os.makedirs(path)
                 lgr.debug("Cache initialized")
-            except:
+            except Exception as e:
                 lgr.error("Failed to initialize cached under %s" % path)
                 raise
         else:
@@ -442,7 +327,7 @@ class ExtractedArchive(object):
         assert (exists(path))
         # create a stamp
         with open(self.stamp_path, 'wb') as f:
-            f.write(assure_bytes(self._archive))
+            f.write(ensure_bytes(self._archive))
         # assert that stamp mtime is not older than archive's directory
         assert (self.is_extracted)
 
@@ -465,7 +350,7 @@ class ExtractedArchive(object):
         path_len = len(path) + (len(os.sep) if not path.endswith(os.sep) else 0)
         for root, dirs, files in os.walk(path):  # TEMP
             for name in files:
-                yield assure_unicode(opj(root, name)[path_len:])
+                yield ensure_unicode(opj(root, name)[path_len:])
 
     def get_leading_directory(self, depth=None, consider=None, exclude=None):
         """Return leading directory of the content within archive
@@ -531,5 +416,5 @@ class ExtractedArchive(object):
         try:
             if self._persistent:
                 self.clean()
-        except:  # MIH: IOError?
+        except Exception as e:  # MIH: IOError?
             pass
