@@ -20,17 +20,27 @@ lgr = logging.getLogger('datalad.support.digests')
 @auto_repr
 class Digester(object):
     """Helper to compute multiple digests in one pass for a file
-
-    Loosely based on snippet by PM 2Ring 2014.10.23
-    http://unix.stackexchange.com/a/163769/55543
-
-    Ideally we should find an efficient way to parallelize this but
-    atm this one is sufficiently speedy
     """
 
+    # Loosely based on snippet by PM 2Ring 2014.10.23
+    # http://unix.stackexchange.com/a/163769/55543
+
+    # Ideally we should find an efficient way to parallelize this but
+    # atm this one is sufficiently speedy
+
     DEFAULT_DIGESTS = ['md5', 'sha1', 'sha256', 'sha512']
 
     def __init__(self, digests=None, blocksize=1 << 16):
+        """
+        Parameters
+        ----------
+        digests : list or None
+          List of any supported algorithm labels, such as md5, sha1, etc.
+          If None, a default set of hashes will be computed (md5, sha1,
+          sha256, sha512).
+        blocksize : int
+          Chunk size (in bytes) by which to consume a file.
+        """
         self._digests = digests or self.DEFAULT_DIGESTS
         self._digest_funcs = [getattr(hashlib, digest) for digest in self._digests]
         self.blocksize = blocksize
@@ -40,6 +50,15 @@ class Digester(object):
         return self._digests
 
     def __call__(self, fpath):
+        """
+        fpath : str
+          File path for which a checksum shall be computed.
+
+        Return
+        ------
+        dict
+          Keys are algorithm labels, and values are checksum strings
+        """
         lgr.debug("Estimating digests for %s" % fpath)
         digests = [x() for x in self._digest_funcs]
         with open(fpath, 'rb') as f:
