@@ -13,6 +13,7 @@ import re
 from os import linesep
 
 
+
 class CommandError(RuntimeError):
     """Thrown if a command call fails.
     """
@@ -137,6 +138,17 @@ class FileNotInRepositoryError(FileNotInAnnexError):
     pass
 
 
+class InvalidGitReferenceError(ValueError):
+    """Thrown if provided git reference is invalid
+    """
+    def __init__(self, ref, *args, **kwargs):
+        super(InvalidGitReferenceError, self).__init__(*args, **kwargs)
+        self.ref = ref
+
+    def __str__(self):
+        return u"Git reference '{}' invalid".format(self.ref)
+
+
 class GitIgnoreError(CommandError):
     """Thrown if a path was ignored by a git command due to .gitignore file
 
@@ -212,6 +224,11 @@ class NoDatasetArgumentFound(InsufficientArgumentsError):
     pass
 
 
+class NoDatasetFound(NoDatasetArgumentFound):
+    """Raised whenever a dataset is required, but none could be determined"""
+    pass
+
+
 class OutOfSpaceError(CommandError):
     """To be raised whenever a command fails if we have no sufficient space
 
@@ -277,6 +294,21 @@ class InvalidAnnexRepositoryError(RuntimeError):
     without init=True"""
 
 
+class DirectModeNoLongerSupportedError(NotImplementedError):
+    """direct mode is no longer supported"""
+
+    def __init__(self, repo, msg=None):
+        super(DirectModeNoLongerSupportedError, self).__init__(
+            ((" " + msg + ", but ") if msg else '')
+            +
+             "direct mode of operation is being deprecated in git-annex and "
+             "no longer supported by DataLad. "
+             "Please use 'git annex upgrade' under %s to upgrade your direct "
+             "mode repository to annex v6 (or later)." % repo.path
+            )
+        self.repo = repo  # might come handy
+
+
 class IncompleteResultsError(RuntimeError):
     """Exception to be raised whenever results are incomplete.
 
@@ -304,6 +336,9 @@ class InstallFailedError(CommandError):
     pass
 
 
+class ConnectionOpenFailedError(CommandError):
+    """Exception to raise whenever opening a network connection fails"""
+    pass
 #
 # Downloaders
 #
@@ -370,4 +405,4 @@ class DataLadWarning(Warning):
 # as a category, so here is a dedicated Warning class
 class OutdatedExternalDependencyWarning(DataLadWarning):
     """Warning "category" to use to report about outdated"""
-    pass
\ No newline at end of file
+    pass
