@@ -11,8 +11,6 @@
 import sys
 import os.path as op
 from os import linesep
-from six import string_types
-from six import binary_type
 
 from distutils.version import LooseVersion
 
@@ -65,7 +63,7 @@ def _get_annex_version():
 
 def _get_git_version():
     """Return version of git we use (might be bundled)"""
-    return __get_git_version(_git_runner)
+    return __get_git_version()
 
 
 def _get_system_git_version():
@@ -93,17 +91,30 @@ def _get_system_ssh_version():
     Annex prior 20170302 was using bundled version, but now would use system one
     if installed
     """
-    try:
-        out, err = _runner.run('ssh -V'.split(),
-                               expect_fail=True, expect_stderr=True)
-        # apparently spits out to err but I wouldn't trust it blindly
-        if err.startswith('OpenSSH'):
-            out = err
-        assert out.startswith('OpenSSH')  # that is the only one we care about atm
-        return out.split(' ', 1)[0].rstrip(',.').split('_')[1]
-    except CommandError as exc:
-        lgr.debug("Could not determine version of ssh available: %s", exc_str(exc))
-        return None
+    out, err = _runner.run('ssh -V'.split(),
+                           expect_fail=True, expect_stderr=True)
+    # apparently spits out to err but I wouldn't trust it blindly
+    if err.startswith('OpenSSH'):
+        out = err
+    assert out.startswith('OpenSSH')  # that is the only one we care about atm
+    return out.split(' ', 1)[0].rstrip(',.').split('_')[1]
+
+
+def _get_system_7z_version():
+    """Return version of 7-Zip"""
+    out, err = _runner.run(
+        ['7z'], expect_fail=True, expect_stderr=True,
+    )
+    # reporting in variable order across platforms
+    # Linux: 7-Zip [64] 16.02
+    # Windows: 7-Zip 19.00 (x86)
+    pieces = out.strip().split(':', maxsplit=1)[0].strip().split()
+    for p in pieces:
+        # the one with the dot is the version
+        if '.' in p:
+            return p
+    lgr.debug("Could not determine version of 7z from stdout. "
+              "stdout: %s, stderr: %s", out, err)
 
 
 class ExternalVersions(object):
@@ -128,6 +139,7 @@ class ExternalVersions(object):
         'cmd:bundled-git': _get_bundled_git_version,
         'cmd:system-git': _get_system_git_version,
         'cmd:system-ssh': _get_system_ssh_version,
+        'cmd:7z': _get_system_7z_version,
     }
     INTERESTING = (
         'appdirs',
@@ -142,9 +154,9 @@ class ExternalVersions(object):
         'msgpack',
         'mutagen',
         'patool',
+        'cmd:7z',
         'requests',
         'scrapy',
-        'six',
         'wrapt',
     )
 
@@ -177,9 +189,9 @@ class ExternalVersions(object):
         if isinstance(version, (tuple, list)):
             #  Generate string representation
             version = ".".join(str(x) for x in version)
-        elif isinstance(version, binary_type):
+        elif isinstance(version, bytes):
             version = version.decode()
-        elif isinstance(version, string_types):
+        elif isinstance(version, str):
             pass
         else:
             version = None
@@ -193,12 +205,13 @@ class ExternalVersions(object):
         # when ran straight in its source code -- fails to discover nipy's version.. TODO
         #if module == 'nipy':
         #    import pdb; pdb.set_trace()
-        if not isinstance(module, string_types):
+        if not isinstance(module, str):
             modname = module.__name__
         else:
             modname = module
             module = None
 
+        lgr.log(5, "Requested to provide version for %s", modname)
         # Early returns None so we do not store prev result for  them
         # and allow users to install things at run time, so later check
         # doesn't pick it up from the _versions
