@@ -10,7 +10,7 @@
 """
 
 import os
-from mock import patch
+from unittest.mock import patch
 
 from ..dochelpers import single_or_plural, borrowdoc, borrowkwargs
 from ..dochelpers import exc_str
