@@ -10,8 +10,8 @@
 
 """
 
-
-from ..support.s3 import get_versioned_url
+from ..support.network import URL
+from ..support.s3 import add_version_to_url, get_versioned_url
 from .utils import use_cassette
 from .utils import ok_startswith
 
@@ -20,16 +20,43 @@ from datalad.tests.utils import skip_if_no_network
 from ..downloaders.tests.utils import get_test_providers
 
 
+def test_add_version_to_url():
+    base_url = "http://ex.com/f.txt"
+    base_url_query = "http://ex.com/f.txt?k=v"
+    for replace in True, False:
+        eq_(add_version_to_url(URL(base_url), "new.id", replace=replace),
+            base_url + "?versionId=new.id")
+
+        eq_(add_version_to_url(URL(base_url_query),
+                               "new.id", replace=replace),
+            base_url_query + "&versionId=new.id")
+
+        expected = "new.id" if replace else "orig.id"
+        eq_(add_version_to_url(URL(base_url + "?versionId=orig.id"),
+                               "new.id",
+                               replace=replace),
+            base_url + "?versionId=" + expected)
+
+        eq_(add_version_to_url(URL(base_url_query + "&versionId=orig.id"),
+                               "new.id",
+                               replace=replace),
+            base_url_query + "&versionId=" + expected)
+
+
 @skip_if_no_network
 @use_cassette('s3_test_version_url')
-def test_version_url():
+def test_get_versioned_url():
     get_test_providers('s3://openfmri/tarballs')  # to verify having credentials to access openfmri via S3
     for url_pref in ('http://openfmri.s3.amazonaws.com', 'https://s3.amazonaws.com/openfmri'):
         eq_(get_versioned_url(url_pref + "/tarballs/ds001_raw.tgz"),
             url_pref + "/tarballs/ds001_raw.tgz?versionId=null")
 
-        eq_(get_versioned_url("http://openfmri.s3.amazonaws.com/tarballs/ds001_raw.tgz?param=1"),
-            "http://openfmri.s3.amazonaws.com/tarballs/ds001_raw.tgz?param=1&versionId=null")
+        eq_(get_versioned_url(url_pref + "/tarballs/ds001_raw.tgz?param=1"),
+            url_pref + "/tarballs/ds001_raw.tgz?param=1&versionId=null")
+
+        # We don't duplicate the version if it already exists.
+        eq_(get_versioned_url(url_pref + "/tarballs/ds001_raw.tgz?versionId=null"),
+            url_pref + "/tarballs/ds001_raw.tgz?versionId=null")
 
     # something is wrong there
     #print(get_versioned_url("http://openfmri.s3.amazonaws.com/ds001/demographics.txt"))
@@ -51,6 +78,13 @@ def test_version_url():
         # so we didn't grab other files along with the same prefix
         ok_startswith(url, 'http://datalad-test0-versioned.s3.amazonaws.com/2versions-removed-recreated.txt?versionId=')
 
+    # Update a versioned URL with a newer version tag.
+    url_3ver = "http://datalad-test0-versioned.s3.amazonaws.com/3versions-allversioned.txt"
+    url_3ver_input = url_3ver + "?versionId=b.qCuh7Sg58VIYj8TVHzbRS97EvejzEl"
+    eq_(get_versioned_url(url_3ver_input), url_3ver_input)
+    eq_(get_versioned_url(url_3ver_input, update=True),
+        url_3ver + "?versionId=Kvuind11HZh._dCPaDAb0OY9dRrQoTMn")
+
 
 @skip_if_no_network
 @use_cassette('s3_test_version_url_deleted')
