@@ -15,10 +15,8 @@ import os, os.path as op
 import shutil
 import sys
 import logging
-from mock import patch
-from six import PY3
-from six import text_type
-import six.moves.builtins as __builtin__
+from unittest.mock import patch
+import builtins
 
 from operator import itemgetter
 from os.path import dirname, normpath, pardir, basename
@@ -66,6 +64,7 @@ from ..utils import unlink
 from ..utils import CMD_MAX_ARG
 from ..utils import create_tree
 from ..utils import never_fail
+from ..utils import Path
 
 from ..support.annexrepo import AnnexRepo
 
@@ -95,9 +94,12 @@ from .utils import ok_startswith
 from .utils import skip_if_no_module
 from .utils import (
     probe_known_failure, skip_known_failure, known_failure, known_failure_v6,
-    known_failure_direct_mode, skip_if,
-    ok_file_has_content
+    skip_if,
+    ok_file_has_content,
+    known_failure_windows,
+    has_symlink_capability,
 )
+from .utils import OBSCURE_FILENAME
 
 
 def test_get_func_kwargs_doc():
@@ -108,7 +110,7 @@ def test_get_func_kwargs_doc():
 
 def test_better_wraps():
     from functools import wraps
-    from inspect import getargspec
+    from datalad.utils import getargspec
 
     def wraps_decorator(func):
         @wraps(func)
@@ -280,14 +282,13 @@ def _check_setup_exceptionhook(interactive):
             except Exception as e:  # RuntimeError:
                 type_, value_, tb_ = sys.exc_info()
             our_exceptionhook(type_, value_, tb_)
-            if PY3:
-                # Happens under tox environment but not in manually crafted
-                # ones -- not yet sure what it is about but --dbg does work
-                # with python3 so lettting it skip for now
-                raise SkipTest(
-                    "TODO: Not clear why in PY3 calls cleanup if we try to "
-                    "access the beast"
-                )
+            # Happens under tox environment but not in manually crafted
+            # ones -- not yet sure what it is about but --dbg does work
+            # with python3 so lettting it skip for now
+            raise SkipTest(
+                "TODO: Not clear why in PY3 calls cleanup if we try to "
+                "access the beast"
+            )
             assert_in('Traceback (most recent call last)', cmo.err)
             assert_in('in _check_setup_exceptionhook', cmo.err)
             if interactive:
@@ -401,6 +402,15 @@ def test_getpwd_symlink(tdir):
         eq_(getpwd(), s2dir)
 
 
+@with_tempfile(mkdir=True)
+def test_chpwd_obscure_name(topdir):
+    path = op.join(topdir, OBSCURE_FILENAME)
+    os.mkdir(path)
+    # Just check that call doesn't fail.
+    with chpwd(path):
+        pass
+
+
 def test_auto_repr():
 
     class WithoutReprClass:
@@ -512,7 +522,9 @@ def test_find_files():
     assert_in(tests_dir, files2)
 
     # now actually matching the path
-    ff3 = find_files('.*/test_.*\.py$', proj_dir, dirs=True)
+    ff3 = find_files(
+        r'.*\\test_.*\.py$' if on_windows else r'.*/test_.*\.py$',
+        proj_dir, dirs=True)
     files3 = list(ff3)
     assert_in(opj(tests_dir, 'test_utils.py'), files3)
     assert_not_in(tests_dir, files3)
@@ -670,6 +682,7 @@ def test_path_():
         eq_(_path_(p, 'd'), 'a/b/c/d')
 
 
+@known_failure_windows
 def test_get_timestamp_suffix():
     # we need to patch temporarily TZ
     import time
@@ -717,9 +730,9 @@ def test_memoized_generator():
 
 
 def test_assure_unicode():
-    ok_(isinstance(assure_unicode("m"), text_type))
-    ok_(isinstance(assure_unicode('grandchild_äöü東'), text_type))
-    ok_(isinstance(assure_unicode(u'grandchild_äöü東'), text_type))
+    ok_(isinstance(assure_unicode("m"), str))
+    ok_(isinstance(assure_unicode('grandchild_äöü東'), str))
+    ok_(isinstance(assure_unicode(u'grandchild_äöü東'), str))
     eq_(assure_unicode('grandchild_äöü東'), u'grandchild_äöü東')
     # now, non-utf8
     # Decoding could be deduced with high confidence when the string is
@@ -732,7 +745,7 @@ def test_assure_unicode():
     eq_(assure_unicode(mom_iso8859, confidence=0.5), u'mamá')
     # but when we mix, it does still guess something allowing to decode:
     mixedin = mom_koi8r + u'東'.encode('iso2022_jp') + u'東'.encode('utf-8')
-    ok_(isinstance(assure_unicode(mixedin), text_type))
+    ok_(isinstance(assure_unicode(mixedin), str))
     # but should fail if we request high confidence result:
     with assert_raises(ValueError):
         assure_unicode(mixedin, confidence=0.9)
@@ -741,6 +754,11 @@ def test_assure_unicode():
     assert assure_unicode(1) is 1
 
 
+def test_pathlib_unicode():
+    eq_(str(Path("a")), u"a")
+    eq_(str(Path(u"β")), u"β")
+
+
 def test_as_unicode():
     eq_(as_unicode('grandchild_äöü東'), u'grandchild_äöü東')
     eq_(as_unicode(None), u"")
@@ -756,6 +774,7 @@ def test_as_unicode():
     assert_in("1 is not of any of known or provided", str(cme.exception))
 
 
+@known_failure_windows
 @with_tempfile(mkdir=True)
 def test_path_prefix(path):
     eq_(get_path_prefix('/d1/d2', '/d1/d2'), '')
@@ -823,6 +842,7 @@ def test_get_dataset_root(path):
         eq_(get_dataset_root(fname), os.curdir)
 
 
+@known_failure_windows
 def test_path_startswith():
     ok_(path_startswith('/a/b', '/a'))
     ok_(path_startswith('/a/b', '/a/b'))
@@ -838,6 +858,7 @@ def test_path_startswith():
     assert_raises(ValueError, path_startswith, '/a/b', 'a')
 
 
+@known_failure_windows
 def test_path_is_subpath():
     ok_(path_is_subpath('/a/b', '/a'))
     ok_(path_is_subpath('/a/b/c', '/a'))
@@ -865,7 +886,7 @@ def test_safe_print():
         if called[0] == 1:
             raise UnicodeEncodeError('crap', u"", 0, 1, 'whatever')
 
-    with patch.object(__builtin__, 'print', _print):
+    with patch.object(builtins, 'print', _print):
         safe_print("bua")
     assert_equal(called[0], 2)
 
@@ -990,37 +1011,25 @@ def test_known_failure_v6():
         assert_raises(AssertionError, failing)
 
 
-def test_known_failure_direct_mode():
-
-    @known_failure_direct_mode
-    def failing():
-        raise AssertionError("Failed")
-
-    from datalad import cfg
-
-    direct = cfg.obtain("datalad.repo.direct")
-    skip = cfg.obtain("datalad.tests.knownfailures.skip")
-    probe = cfg.obtain("datalad.tests.knownfailures.probe")
-
-    if direct:
-        if skip:
-            # skipping takes precedence over probing
-            failing()
-        elif probe:
-            # if we probe a known failure it's okay to fail:
-            failing()
-        else:
-            # not skipping and not probing results in the original failure:
-            assert_raises(AssertionError, failing)
-
-    else:
-        # behaves as if it wasn't decorated at all, no matter what
-        assert_raises(AssertionError, failing)
-
-
 from datalad.utils import read_csv_lines
 
 
+def test_known_failure_direct_mode():
+    # Decorator is deprecated now and that is what we check
+    from .utils import known_failure_direct_mode
+
+    x = []
+    with swallow_logs(new_level=logging.WARNING) as cml:
+        @known_failure_direct_mode
+        def failing():
+            x.append('ok')
+            raise AssertionError("Failed")
+
+        assert_raises(AssertionError, failing)  # nothing is swallowed
+        eq_(x, ['ok'])  # everything runs
+        assert_in("Direct mode support is deprecated", cml.out)
+
+
 @with_tempfile(content="h1 h2\nv1 2\nv2 3")
 def test_read_csv_lines_basic(infile):
     # Just a basic test, next one with unicode
@@ -1131,6 +1140,8 @@ def test_line_profile():
 
 @with_tempfile(mkdir=True)
 def test_dlabspath(path):
+    if not has_symlink_capability():
+        raise SkipTest
     # initially ran into on OSX https://github.com/datalad/datalad/issues/2406
     opath = opj(path, "origin")
     os.makedirs(opath)
