@@ -13,14 +13,12 @@
 __docformat__ = 'restructuredtext'
 
 from abc import ABCMeta, abstractmethod
-from six import add_metaclass
 
 from ..utils import auto_repr
 
 
 @auto_repr
-@add_metaclass(ABCMeta)
-class InteractiveUI(object):
+class InteractiveUI(object, metaclass=ABCMeta):
     """Semi-abstract class for interfaces to implement interactive UI"""
 
     @abstractmethod
