@@ -10,11 +10,10 @@
 
 __docformat__ = 'restructuredtext'
 
-from six import PY2
-from six.moves import StringIO
-import six.moves.builtins as __builtin__
+from io import StringIO
+import builtins
 
-from mock import (
+from unittest.mock import (
     call,
     patch,
 )
@@ -33,7 +32,7 @@ from datalad.ui.progressbars import progressbars
 
 def patch_input(**kwargs):
     """A helper to provide mocked cm patching input function which was renamed in PY3"""
-    return patch.object(__builtin__, 'raw_input' if PY2 else 'input', **kwargs)
+    return patch.object(builtins, 'input', **kwargs)
 
 
 def patch_getpass(**kwargs):
@@ -102,6 +101,13 @@ def test_hidden_doubleentry():
         eq_(response, 'ab')
         gpcm.assert_has_calls([call('?: '), call('? (repeat): ')])
 
+    # explicitly request no repeats
+    with patch_getpass(return_value='ab') as gpcm:
+        response = ui.question(
+            "?", hidden=True, repeat=False)
+        eq_(response, 'ab')
+        gpcm.assert_has_calls([call('?: ')])
+
 
 def _test_progress_bar(backend, len, increment):
     out = StringIO()
@@ -110,6 +116,8 @@ def _test_progress_bar(backend, len, increment):
         'label', fill_str, total=10, backend=backend)
     pb.start()
     # we can't increment 11 times
+    SILENT_BACKENDS = ('annex-remote', 'silent', 'none')
+    ONLY_THE_END_BACKENDS = ('log',)
     for x in range(11):
         if not (increment and x == 0):
             # do not increment on 0
@@ -121,15 +129,17 @@ def _test_progress_bar(backend, len, increment):
         # or just force the refresh
         pb.refresh()
         pstr = out.getvalue()
-        if backend not in ('annex-remote', 'silent'):  # no str repr
+        if backend not in SILENT_BACKENDS + ONLY_THE_END_BACKENDS:  # no str repr
             ok_startswith(pstr.lstrip('\r'), 'label:')
             assert_re_in(r'.*\b%d%%.*' % (10*x), pstr)
         if backend == 'progressbar':
             assert_in('ETA', pstr)
     pb.finish()
-    if backend not in ('annex-remote', 'silent'):
+    output = out.getvalue()
+    if backend not in SILENT_BACKENDS:
         # returns back and there is no spurious newline
-        ok_endswith(out.getvalue(), '\r')
+        if output:
+            ok_endswith(output, '\r')
 
 
 def test_progress_bar():
